<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DataSource\DataSet as DataSet_Source;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class AcoesReceitaPorServico extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $pratoService;
    protected $newCode;
    protected $environment;

    protected $parameterBagColumns = array();
    protected $dataSourceManager;

    public function __construct(
        $crudService, $parameterBag, $dataSourceManager,
        \Teknisa\PLA\Modules\Prato\Services\PratoService $pratoService,
        \Teknisa\GER\Utilities\Environment $environment,
        \Teknisa\GER\Services\NewCode $newCode
    ) {
        parent::__construct($crudService, $parameterBag);
        $this->pratoService      = $pratoService;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
    }

    public function replicaReceitaPorServico(
        \Zeedhi\Framework\DTO\Request\Row $request,
        \Zeedhi\Framework\DTO\Response $response
    ) {
        try {
            $row = $request->getRow();
            $tpReceita = $row['RECEITAORIGEM'];
            $filialO = $row['FILIALORIGEM'];
            if ($filialO === 'X') $filialO = null;
            $servicoO = $row['SERVICOORIGEM'];
            if ($servicoO === 'X') $servicoO = null;

            $pratos = $row['LINHASPRATO'];
            $servicos = $row['LINHASSERVICO'];
            
            $nrorg = $this->environment->getNrOrgTrab();

            $this->pratoService->iniciaTransacao();

            foreach ($servicos as $servico) {
                $percentual = $servico['PERCENTUAL'];

                if (!$percentual || $percentual <= 0) {
                    throw new \Exception('O valor do percentual deve ser maior que zero.');
                } else {
                    $percentual = str_replace(',', '.', $percentual);
                }

                $filialD = $servico['CDFILIAL'];
                $servicoD = $servico['CDSERVFILI'];

                foreach ($pratos as $prato) {
                    // EXCLUSÕES
                    // EXCLUI PRODUTOS DA RECEITA POR SERVIÇO SELECIONADO
                    $produtosReceita = $this->retornaReceita(
                        'receserv', $prato['CDPRATO'], $prato['CDFILAUXPRAT'],
                        $filialD, $servicoD
                    );
                    
                    if ($produtosReceita) {
                        foreach ($produtosReceita as $produtoReceita) {
                            $this->pratoService->excluiReceita($produtoReceita);
                        }
                    }

                    // EXCLUI DADOS DA RECEITA POR SERVIÇO SELECIONADO
                    $dados = $this->retornaReceita(
                        'pratserv', $prato['CDPRATO'], $prato['CDFILAUXPRAT'],
                        $filialD, $servicoD
                    );

                    if ($dados) {
                        $this->pratoService->excluiDadosReceita($dados);
                    }

                    // EXCLUI GRAMAGEM DO PRATO DA RECEITA POR SERVIÇO SELECIONADO
                    $gramagem = $this->pratoService->retornaPratoGramagem(
                        $prato['CDPRATO'], $prato['CDFILAUXPRAT'],
                        $filialD, $servicoD
                    );
                    if ($gramagem) {
                        $this->pratoService->excluiPratoGramagem($gramagem);
                    }

                    //INSERÇÕES
                    // TIRA DA RECEITA DE ORIGEM OS PRODUTOS QUE NÃO SÃO ATIVOS NA FILIAL
                    // DE DESTINO
                    switch ($tpReceita) {
                    case 'P':
                        $receitaO = $this->retornaReceita('recepadr', $prato['CDPRATO'], $prato['CDFILAUXPRAT']);
                        $dadosReceitaO = $this->retornaReceita('pratpadr', $prato['CDPRATO'], $prato['CDFILAUXPRAT']);
                        break;
                    case 'L':
                        $receitaO = $this->retornaReceita('recefili', $prato['CDPRATO'], $prato['CDFILAUXPRAT'], $filialO);
                        $dadosReceitaO = $this->retornaReceita('pratfili', $prato['CDPRATO'], $prato['CDFILAUXPRAT'], $filialO);
                        break;
                    case 'S':
                        $receitaO = $this->retornaReceita('receserv', $prato['CDPRATO'], $prato['CDFILAUXPRAT'], $filialO, $servicoO);
                        $dadosReceitaO = $this->retornaReceita('pratserv', $prato['CDPRATO'], $prato['CDFILAUXPRAT'], $filialO, $servicoO);
                        break;
                    }

                    //Insere Receita
                    if ($receitaO) {
                        foreach ($receitaO as $prodReceitaO) {
                            $produtoInserir = array(
                                'NRORG' => $nrorg,
                                '__is_new' => true,
                                'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                                'CDPRATO' => $prato['CDPRATO'],
                                'CDFILIAL' => $filialD,
                                'CDSERVFILI' => $servicoD,
                                'CDPRODUTO' => $prodReceitaO['CDPRODUTO'],
                                'SGUNIDADE' => $prodReceitaO['SGUNIDADE']
                            );

                            switch ($tpReceita) {
                            case 'P':
                                $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODPRAT']   * $percentual / 100;
                                $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCPR'];
                                $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORPR'];
                                $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTAPR']    * $percentual / 100;
                                $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTPR']  * $percentual / 100;
                                $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTPR'];
                                $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRPR'] * $percentual / 100;
                                $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTAPRUNR'] * $percentual / 100;
                                $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPPR'];
                                $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASEPR'];
                                $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXPR'];
                                $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRP'];
                                $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODPADR'];
                                $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODPADR'];
                                $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                                break;
                            case 'L':
                                $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODRECLOC'] * $percentual / 100;
                                $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCLO'];
                                $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORLO'];
                                $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTALO']    * $percentual / 100;
                                $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTLO']  * $percentual / 100;
                                $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTLO'];
                                $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRLO'] * $percentual / 100;
                                $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTALOUNR'] * $percentual / 100;
                                $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPLO'];
                                $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASELO'];
                                $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXLO'];
                                $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRF'];
                                $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODFILI'];
                                $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODFILI'];
                                $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                                break;
                            case 'S':
                                $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODRECSER'] * $percentual / 100;
                                $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCSE'];
                                $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORSE'];
                                $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTASE']    * $percentual / 100;
                                $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTSE']  * $percentual / 100;
                                $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTSE'];
                                $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRSE'] * $percentual / 100;
                                $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTASEUNR'] * $percentual / 100;
                                $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPSE'];
                                $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASESE'];
                                $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXSE'];
                                $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRS'];
                                $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODSERV'];
                                $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODSERV'];
                                $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                                break;
                            }
                            $produtoInserir['VRPERCDEGELO'] = $prodReceitaO['VRPERCDEGELO'];

                            $requestInsertProduto
                                = new \Zeedhi\Framework\DTO\Request\Row(
                                $produtoInserir, '', '', ''
                            );
                            $produtoInserir = $this->crudService->getRowsToSave(
                                $requestInsertProduto, "receserv"
                            );
                            $this->crudService->save(
                                "receserv", $produtoInserir
                            );
                        }
                    }

                    // Insere dados da receita
                    if ($dadosReceitaO && count($dadosReceitaO) > 0) {
                        $pratPadrInserir = array(
                            'NRORG'        => $nrorg,
                            '__is_new'     => true,
                            'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                            'CDPRATO'      => $prato['CDPRATO'],
                            'CDFILIAL'     => $filialD,
                            'CDSERVFILI'   => $servicoD,
                            'DSPRATO'      => ""
                        );
                        $dadosReceitaO = $dadosReceitaO[0];

                        switch ($tpReceita) {
                        case 'P':
                            $pratPadrInserir['NRCORPRISERV']   = $dadosReceitaO['NRCORPRIPRAT'];
                            $pratPadrInserir['NRCORSEQSERV']   = $dadosReceitaO['NRCORSEQPRAT'];
                            $pratPadrInserir['NRCORAUXSERV']   = $dadosReceitaO['NRCORAUXPRAT'];
                            $pratPadrInserir['QTPADRAOSERV']   = $dadosReceitaO['QTPADRAOPRAT'] * $percentual / 100;
                            $pratPadrInserir['QTCONTAISERV']   = $dadosReceitaO['QTCONTAIPRAT'];
                            $pratPadrInserir['HRVALCONSERV']   = $dadosReceitaO['HRVALCONPRAT'];
                            $pratPadrInserir['NMFANTASIASE']   = $dadosReceitaO['NMFANTASIAPR'];
                            $pratPadrInserir['QTCARBOPRSER']   = $dadosReceitaO['QTCARBOPRPAD'];
                            $pratPadrInserir['QTLIPIDPRSER']   = $dadosReceitaO['QTLIPIDPRPAD'];
                            $pratPadrInserir['QTPROTEPRSER']   = $dadosReceitaO['QTPROTEPRPAD'];
                            $pratPadrInserir['QTCALORPRSER']   = $dadosReceitaO['QTCALORPRPAD'];
                            $pratPadrInserir['QTALCOOPRSER']   = $dadosReceitaO['QTALCOOPRPAD'];
                            $pratPadrInserir['VRPEMINSERV']    = $dadosReceitaO['VRPEMINPADR'];
                            $pratPadrInserir['QTAMOSTRSERV']   = $dadosReceitaO['QTAMOSTRPADR'];
                            $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                            $pratPadrInserir['QTLIMSEGSERV']   = $dadosReceitaO['QTLIMSEGPADR'];
                            $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                            $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                            $pratPadrInserir['IDFINALSERV']    = $dadosReceitaO['IDFINALPADR'];
                            break;
                        case 'L':
                            $pratPadrInserir['NRCORPRISERV']   = $dadosReceitaO['NRCORPRIFILI'];
                            $pratPadrInserir['NRCORSEQSERV']   = $dadosReceitaO['NRCORSEQFILI'];
                            $pratPadrInserir['NRCORAUXSERV']   = $dadosReceitaO['NRCORAUXFILI'];
                            $pratPadrInserir['QTPADRAOSERV']   = $dadosReceitaO['QTPADRAOFILI'] * $percentual / 100;
                            $pratPadrInserir['QTCONTAISERV']   = $dadosReceitaO['QTCONTAIFILI'];
                            $pratPadrInserir['HRVALCONSERV']   = $dadosReceitaO['HRVALCONFILI'];
                            $pratPadrInserir['NMFANTASIASE']   = $dadosReceitaO['NMFANTASIAPF'];
                            $pratPadrInserir['QTCARBOPRSER']   = $dadosReceitaO['QTCARBOPRFIL'];
                            $pratPadrInserir['QTLIPIDPRSER']   = $dadosReceitaO['QTLIPIDPRFIL'];
                            $pratPadrInserir['QTPROTEPRSER']   = $dadosReceitaO['QTPROTEPRFIL'];
                            $pratPadrInserir['QTCALORPRSER']   = $dadosReceitaO['QTCALORPRFIL'];
                            $pratPadrInserir['QTALCOOPRSER']   = $dadosReceitaO['QTALCOOPRFIL'];
                            $pratPadrInserir['VRPEMINSERV']    = $dadosReceitaO['VRPEMINFILI'];
                            $pratPadrInserir['QTAMOSTRSERV']   = $dadosReceitaO['QTAMOSTRFILI'];
                            $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                            $pratPadrInserir['QTLIMSEGSERV']   = $dadosReceitaO['QTLIMSEGFILI'];
                            $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                            $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                            $pratPadrInserir['IDFINALSERV']    = $dadosReceitaO['IDFINALFILI'];
                            break;
                        case 'S':
                            $pratPadrInserir['NRCORPRISERV']   = $dadosReceitaO['NRCORPRISERV'];
                            $pratPadrInserir['NRCORSEQSERV']   = $dadosReceitaO['NRCORSEQSERV'];
                            $pratPadrInserir['NRCORAUXSERV']   = $dadosReceitaO['NRCORAUXSERV'];
                            $pratPadrInserir['QTPADRAOSERV']   = $dadosReceitaO['QTPADRAOSERV'] * $percentual / 100;
                            $pratPadrInserir['QTCONTAISERV']   = $dadosReceitaO['QTCONTAISERV'];
                            $pratPadrInserir['HRVALCONSERV']   = $dadosReceitaO['HRVALCONSERV'];
                            $pratPadrInserir['NMFANTASIASE']   = $dadosReceitaO['NMFANTASIASE'];
                            $pratPadrInserir['QTCARBOPRSER']   = $dadosReceitaO['QTCARBOPRSER'];
                            $pratPadrInserir['QTLIPIDPRSER']   = $dadosReceitaO['QTLIPIDPRSER'];
                            $pratPadrInserir['QTPROTEPRSER']   = $dadosReceitaO['QTPROTEPRSER'];
                            $pratPadrInserir['QTCALORPRSER']   = $dadosReceitaO['QTCALORPRSER'];
                            $pratPadrInserir['QTALCOOPRSER']   = $dadosReceitaO['QTALCOOPRSER'];
                            $pratPadrInserir['VRPEMINSERV']    = $dadosReceitaO['VRPEMINSERV'];
                            $pratPadrInserir['QTAMOSTRSERV']   = $dadosReceitaO['QTAMOSTRSERV'];
                            $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                            $pratPadrInserir['QTLIMSEGSERV']   = $dadosReceitaO['QTLIMSEGSERV'];
                            $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                            $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                            $pratPadrInserir['IDFINALSERV']    = $dadosReceitaO['IDFINALSERV'];
                            break;
                        }

                        $requestInsertPrat
                            = new \Zeedhi\Framework\DTO\Request\Row(
                            $pratPadrInserir, '', '', ''
                        );
                        
                        $pratPadrInserir = $this->crudService->getRowsToSave(
                            $requestInsertPrat, "pratserv"
                        );
                        $this->crudService->save("pratserv", $pratPadrInserir);
                    }

                    // Insere Gramagem por Prato
                    $gramagPratoInserir = array(
                        'NRORG' => $nrorg,
                        '__is_new' => true,
                        'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                        'CDPRATO' => $prato['CDPRATO'],
                        'CDFILIAL' => $filialD,
                        'CDSERVFILI' => $servicoD,
                        'IDORIGEMRECE' => $tpReceita === 'L' ? 'F' : $tpReceita,
                        'CDFILIALORI' => $filialO,
                        'CDSERVFILIORI' => $servicoO,
                        'VRPERCENTUAL' => $percentual
                    );

                    $requestInsertPrat = new \Zeedhi\Framework\DTO\Request\Row(
                        $gramagPratoInserir, '', '', ''
                    );
                    $gramagPratoInserir = $this->crudService->getRowsToSave(
                        $requestInsertPrat, "pratogramag"
                    );
                    $this->crudService->save("pratogramag", $gramagPratoInserir);

                }
            }

            $response->addDataset(new DataSet_Source('RETORNO', array(true)));
            $this->pratoService->commit();
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function percentualReceitaPorServico(
        \Zeedhi\Framework\DTO\Request\Row $request,
        \Zeedhi\Framework\DTO\Response $response
    ) {
        try {
            $row = $request->getRow();
            
            $servicos = $row['LINHASSERVICO'];
            $filialD = $row['CDFILIALD'];
            $prato = $row['CDPRATOO'];
            $filAuxPrat = $row['CDFILAUXPRATO'];

            $nrorg = $this->environment->getNrOrgTrab();

            $this->pratoService->iniciaTransacao();

            foreach ($servicos as $servico) {
                $percentual = $servico['PERCENTUAL'];
                if(isset($servico['TPRECEITAO']) && $servico['TPRECEITAO']){
                    $tpReceita = $servico['TPRECEITAO'];
                } else {
                    $tpReceita = 'P';
                }
                
                
                if ($tpReceita === 'P') {
                    $filialO = 'X';
                    $servicoO = 'X';
                } else if ($tpReceita === 'L') {
                    $filialO = $servico['CDFILIALO'];
                    $servicoO = 'X';
                } else {
                    $filialO = $servico['CDFILIALO'];
                    $servicoO = $servico['CDSERVFILIO'];
                }

                $servicoD = $servico['CDSERVFILI'];

                // EXCLUSÕES
                // EXCLUI PRODUTOS DA RECEITA POR SERVIÇO SELECIONADO
                $produtosReceita = $this->retornaReceita(
                    'receserv', $prato, $filAuxPrat, $filialD, $servicoD
                );

                if ($produtosReceita) {
                    foreach ($produtosReceita as $produtoReceita) {
                        $this->pratoService->excluiReceita($produtoReceita);
                    }
                }

                // EXCLUI DADOS DA RECEITA POR SERVIÇO SELECIONADO
                $dados = $this->retornaReceita(
                    'pratserv', $prato, $filAuxPrat, $filialD, $servicoD
                );

                if ($dados) {
                    $this->pratoService->excluiDadosReceita($dados);
                }

                // EXCLUI GRAMAGEM DO PRATO DA RECEITA POR SERVIÇO SELECIONADO
                $gramagem = $this->pratoService->retornaPratoGramagem(
                    $prato, $filAuxPrat, $filialD, $servicoD
                );

                if ($gramagem) {
                    $this->pratoService->excluiPratoGramagem($gramagem);
                }

                //INSERÇÕES
                // TIRA DA RECEITA DE ORIGEM OS PRODUTOS QUE NÃO SÃO ATIVOS NA FILIAL
                // DE DESTINO
                switch ($tpReceita) {
                case 'P':
                    $receitaO = $this->retornaReceita('recepadr', $prato, $filAuxPrat);
                    $dadosReceitaO = $this->retornaReceita('pratpadr', $prato, $filAuxPrat);
                    break;
                case 'L':
                    $receitaO = $this->retornaReceita('recefili', $prato, $filAuxPrat, $filialO);
                    $dadosReceitaO = $this->retornaReceita('pratfili', $prato, $filAuxPrat, $filialO);
                    break;
                case 'S':
                    $receitaO = $this->retornaReceita('receserv', $prato, $filAuxPrat, $filialO, $servicoO);
                    $dadosReceitaO = $this->retornaReceita('pratserv', $prato, $filAuxPrat, $filialO, $servicoO);
                    break;
                }
                
                //Insere Receita
                if ($receitaO) {
                    foreach ($receitaO as $prodReceitaO) {
                        $produtoInserir = array(
                            'NRORG' => $nrorg,
                            '__is_new' => true,
                            'CDFILAUXPRAT' => $filAuxPrat,
                            'CDPRATO' => $prato,
                            'CDFILIAL' => $filialD,
                            'CDSERVFILI' => $servicoD,
                            'CDPRODUTO' => $prodReceitaO['CDPRODUTO'],
                            'SGUNIDADE' => $prodReceitaO['SGUNIDADE']
                        );

                        switch ($tpReceita) {
                        case 'P':
                            $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODPRAT'] * $percentual / 100;
                            $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCPR'];
                            $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORPR'];
                            $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTAPR'] * $percentual / 100;
                            $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTPR'] * $percentual / 100;
                            $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTPR'];
                            $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRPR'] * $percentual / 100;
                            $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTAPRUNR'] * $percentual / 100;
                            $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPPR'];
                            $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASEPR'];
                            $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXPR'];
                            $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRP'];
                            $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODPADR'];
                            $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODPADR'];
                            $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                            break;
                        case 'L':
                            $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODRECLOC'] * $percentual / 100;
                            $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCLO'];
                            $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORLO'];
                            $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTALO'] * $percentual / 100;
                            $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTLO'] * $percentual / 100;
                            $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTLO'];
                            $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRLO'] * $percentual / 100;
                            $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTALOUNR'] * $percentual / 100;
                            $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPLO'];
                            $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASELO'];
                            $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXLO'];
                            $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRF'];
                            $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODFILI'];
                            $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODFILI'];
                            $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                            break;
                        case 'S':
                            $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODRECSER'] * $percentual / 100;
                            $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCSE'];
                            $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORSE'];
                            $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTASE'] * $percentual / 100;
                            $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTSE'] * $percentual / 100;
                            $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTSE'];
                            $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRSE'] * $percentual / 100;
                            $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTASEUNR'] * $percentual / 100;
                            $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPSE'];
                            $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASESE'];
                            $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXSE'];
                            $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRS'];
                            $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODSERV'];
                            $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODSERV'];
                            $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                            break;
                        }
                        $produtoInserir['VRPERCDEGELO'] = $prodReceitaO['VRPERCDEGELO'];
                        $requestInsertProduto
                            = new \Zeedhi\Framework\DTO\Request\Row(
                            $produtoInserir, '', '', ''
                        );
                        $produtoInserir = $this->crudService->getRowsToSave(
                            $requestInsertProduto, "receserv"
                        );

                        $this->crudService->save(
                            "receserv", $produtoInserir
                        );
                    }
                }

                // Insere dados da receita
                if ($dadosReceitaO && count($dadosReceitaO) > 0) {
                    $pratPadrInserir = array(
                        'NRORG' => $nrorg,
                        '__is_new' => true,
                        'CDFILAUXPRAT' => $filAuxPrat,
                        'CDPRATO' => $prato,
                        'CDFILIAL' => $filialD,
                        'CDSERVFILI' => $servicoD,
                        'DSPRATO' => ''
                    );
                    $dadosReceitaO = $dadosReceitaO[0];

                    switch ($tpReceita) {
                    case 'P':
                        $pratPadrInserir['NRCORPRISERV']   = $dadosReceitaO['NRCORPRIPRAT'];
                        $pratPadrInserir['NRCORSEQSERV']   = $dadosReceitaO['NRCORSEQPRAT'];
                        $pratPadrInserir['NRCORAUXSERV']   = $dadosReceitaO['NRCORAUXPRAT'];
                        $pratPadrInserir['QTPADRAOSERV']   = $dadosReceitaO['QTPADRAOPRAT'] * $percentual / 100;
                        $pratPadrInserir['QTCONTAISERV']   = $dadosReceitaO['QTCONTAIPRAT'];
                        $pratPadrInserir['HRVALCONSERV']   = $dadosReceitaO['HRVALCONPRAT'];
                        $pratPadrInserir['NMFANTASIASE']   = $dadosReceitaO['NMFANTASIAPR'];
                        $pratPadrInserir['QTCARBOPRSER']   = $dadosReceitaO['QTCARBOPRPAD'];
                        $pratPadrInserir['QTLIPIDPRSER']   = $dadosReceitaO['QTLIPIDPRPAD'];
                        $pratPadrInserir['QTPROTEPRSER']   = $dadosReceitaO['QTPROTEPRPAD'];
                        $pratPadrInserir['QTCALORPRSER']   = $dadosReceitaO['QTCALORPRPAD'];
                        $pratPadrInserir['QTALCOOPRSER']   = $dadosReceitaO['QTALCOOPRPAD'];
                        $pratPadrInserir['VRPEMINSERV']    = $dadosReceitaO['VRPEMINPADR'];
                        $pratPadrInserir['QTAMOSTRSERV']   = $dadosReceitaO['QTAMOSTRPADR'];
                        $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                        $pratPadrInserir['QTLIMSEGSERV']   = $dadosReceitaO['QTLIMSEGPADR'];
                        $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                        $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                        $pratPadrInserir['IDFINALSERV']    = $dadosReceitaO['IDFINALPADR'];
                        break;
                    case 'L':
                        $pratPadrInserir['NRCORPRISERV']   = $dadosReceitaO['NRCORPRIFILI'];
                        $pratPadrInserir['NRCORSEQSERV']   = $dadosReceitaO['NRCORSEQFILI'];
                        $pratPadrInserir['NRCORAUXSERV']   = $dadosReceitaO['NRCORAUXFILI'];
                        $pratPadrInserir['QTPADRAOSERV']   = $dadosReceitaO['QTPADRAOFILI'] * $percentual / 100;
                        $pratPadrInserir['QTCONTAISERV']   = $dadosReceitaO['QTCONTAIFILI'];
                        $pratPadrInserir['HRVALCONSERV']   = $dadosReceitaO['HRVALCONFILI'];
                        $pratPadrInserir['NMFANTASIASE']   = $dadosReceitaO['NMFANTASIAPF'];
                        $pratPadrInserir['QTCARBOPRSER']   = $dadosReceitaO['QTCARBOPRFIL'];
                        $pratPadrInserir['QTLIPIDPRSER']   = $dadosReceitaO['QTLIPIDPRFIL'];
                        $pratPadrInserir['QTPROTEPRSER']   = $dadosReceitaO['QTPROTEPRFIL'];
                        $pratPadrInserir['QTCALORPRSER']   = $dadosReceitaO['QTCALORPRFIL'];
                        $pratPadrInserir['QTALCOOPRSER']   = $dadosReceitaO['QTALCOOPRFIL'];
                        $pratPadrInserir['VRPEMINSERV']    = $dadosReceitaO['VRPEMINFILI'];
                        $pratPadrInserir['QTAMOSTRSERV']   = $dadosReceitaO['QTAMOSTRFILI'];
                        $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                        $pratPadrInserir['QTLIMSEGSERV']   = $dadosReceitaO['QTLIMSEGFILI'];
                        $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                        $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                        $pratPadrInserir['IDFINALSERV']    = $dadosReceitaO['IDFINALFILI'];
                        break;
                    case 'S':
                        $pratPadrInserir['NRCORPRISERV']   = $dadosReceitaO['NRCORPRISERV'];
                        $pratPadrInserir['NRCORSEQSERV']   = $dadosReceitaO['NRCORSEQSERV'];
                        $pratPadrInserir['NRCORAUXSERV']   = $dadosReceitaO['NRCORAUXSERV'];
                        $pratPadrInserir['QTPADRAOSERV']   = $dadosReceitaO['QTPADRAOSERV'] * $percentual / 100;
                        $pratPadrInserir['QTCONTAISERV']   = $dadosReceitaO['QTCONTAISERV'];
                        $pratPadrInserir['HRVALCONSERV']   = $dadosReceitaO['HRVALCONSERV'];
                        $pratPadrInserir['NMFANTASIASE']   = $dadosReceitaO['NMFANTASIASE'];
                        $pratPadrInserir['QTCARBOPRSER']   = $dadosReceitaO['QTCARBOPRSER'];
                        $pratPadrInserir['QTLIPIDPRSER']   = $dadosReceitaO['QTLIPIDPRSER'];
                        $pratPadrInserir['QTPROTEPRSER']   = $dadosReceitaO['QTPROTEPRSER'];
                        $pratPadrInserir['QTCALORPRSER']   = $dadosReceitaO['QTCALORPRSER'];
                        $pratPadrInserir['QTALCOOPRSER']   = $dadosReceitaO['QTALCOOPRSER'];
                        $pratPadrInserir['VRPEMINSERV']    = $dadosReceitaO['VRPEMINSERV'];
                        $pratPadrInserir['QTAMOSTRSERV']   = $dadosReceitaO['QTAMOSTRSERV'];
                        $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                        $pratPadrInserir['QTLIMSEGSERV']   = $dadosReceitaO['QTLIMSEGSERV'];
                        $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                        $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                        $pratPadrInserir['IDFINALSERV']    = $dadosReceitaO['IDFINALSERV'];
                        break;
                    }

                    $requestInsertPrat
                        = new \Zeedhi\Framework\DTO\Request\Row(
                        $pratPadrInserir, '', '', ''
                    );
                    $pratPadrInserir = $this->crudService->getRowsToSave(
                        $requestInsertPrat, "pratserv"
                    );
                    $this->crudService->save("pratserv", $pratPadrInserir);
                }

                // Insere Gramagem por Prato
                $gramagPratoInserir = array(
                    'NRORG' => $nrorg,
                    '__is_new' => true,
                    'CDFILAUXPRAT' => $filAuxPrat,
                    'CDPRATO' => $prato,
                    'CDFILIAL' => $filialD,
                    'CDSERVFILI' => $servicoD,
                    'IDORIGEMRECE' => $tpReceita === 'L' ? 'F' : $tpReceita,
                    'CDFILIALORI' => $filialO,
                    'CDSERVFILIORI' => $servicoO,
                    'VRPERCENTUAL' => $percentual
                );

                $requestInsertPrat = new \Zeedhi\Framework\DTO\Request\Row(
                    $gramagPratoInserir, '', '', ''
                );
                $gramagPratoInserir = $this->crudService->getRowsToSave(
                    $requestInsertPrat, "pratogramag"
                );
                $this->crudService->save("pratogramag", $gramagPratoInserir);
            }

            $response->addDataset(new DataSet_Source('RETORNO', array(true)));
            $this->pratoService->commit();
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
        }
    } 
    private function retornaReceita($dataSourceName, $prato, $filAuxPrat, $filial = null, $servico = null, $produto = null) {
        $conditions = array(
            array(
                "columnName" => "CDPRATO",
                "operator" => "=",
                "value" => $prato
            ),
            array(
                "columnName" => "CDFILAUXPRAT",
                "operator" => "=",
                "value" => $filAuxPrat
            )
        );
        if($servico || $filial){
            if($servico){
                array_push($conditions, 
                    array(
                        "columnName" => "CDSERVFILI",
                        "operator" => "=",
                        "value" => $servico
                    )
                );
            }
            array_push($conditions, 
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $filial
                )
            );
        }
        if($produto) { 
            array_push($conditions, 
                array(
                    "columnName" => "CDPRODUTO",
                    "operator" => "=",
                    "value" => $produto
                )
            );
        }
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($dataSourceName, $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        return $dataSet->getRows();
    }
}