<?php
namespace Teknisa\PLA\Modules\Prato\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Utilities\DataSourceUtil;

class PratoService extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $entityManager;
    
    protected $TAM_CODIGO = 'O tamanho do código está superior a 10 caracteres.';
    protected $TAM_CODIGO_PRATO = 'Código do prato com nível inválido.';
    protected $CDPRATO_N_INFO = 'Código do prato não foi informado.';
    protected $MASC_N_DEFINIDA = 'Mascara do prato não definida.';
    
    protected $TAM_PRATO = 'O tamanho do nome do prato está superior a 40 caracteres.';
    protected $NMPRATO_N_INFO = 'Nome do prato não foi informado.';
    
    protected $TAM_UNIDADE = 'O tamanho da unidade de medida está superior a 10 caracteres.';
    protected $UN_PRATO_N_INFO = 'Unidade do prato não informada.';
    protected $UN_PRATO_N_EXISTE = 'Unidade do prato não cadastrado.';
    
    protected $QTD_PERDA_INVALIDA = 'Quantidade de perda inválida.';
    protected $QTD_PROD_INVALIDA = 'Quantidade do produto inválida.';
    protected $QTD_SUPERIOR = 'Quantidade da perda deve estar entre 0 e 99.';
    protected $QTD_PROD_N_INFO = 'Quantidade do produto não informada.';
    
    protected $PRODUTO_N_EXISTE = 'Produto não cadastrado ou não é um produto de estoque.';
    
    protected $UN_PRODUTO_N_INFO = 'Unidade do produto não foi informado.';
    protected $UN_PROD_N_EXISTE = 'Unidade do produto não cadastrada.';
    protected $UN_PROD_DIF = 'Unidade do produto informada não é a mesma unidade de medida do cadastro do produto.';
    protected $PROD_N_INFO = 'O produto deve ser informado quando o prato for de nível completo.';
    protected $PROD_JA_INFO = 'O produto já foi informado para um mesmo prato anteriormente.';

    public function __construct(EntityManager $entityManager, Environment $environment) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }

    public function retornaPratos($filial, $addWhere){
        $sql = "SELECT P.CDFILIAL, P.IDATIVO, P.CDNVPRATO,
                      CASE
                            WHEN P.CDPRATONIVANTE = '0' AND P.CDNVPRATO = 1
                            THEN NULL
                            ELSE P.CDPRATONIVANTE
                        END AS CDPRATONIVANTE,
                        P.CDPRATO AS CDPRATO_AUX,
                        FORMATPRATMASK(P.CDPRATO) AS CDPRATOFORMATADO,
                        P.CDPRATO, P.CDFILAUXPRAT, P.NMPRATO,
                        DECODE(P.CDFILAUXPRAT, 'P', 'P', 'L') AS IDTIPO,
                        P.SGUNIDADE, P.DSUNIDADE, P.NRFORMAPREPARO, P.NMFORMAPREPARO,
                        P.NRORG, P.VRARREDTRANS, P.QTTEMPOPRAT, P.IDFREQUENCIA,
                        CASE
                        WHEN CHILDCOUNT > 0
                        THEN 'true'
                        ELSE 'false'
                      END AS HASCHILD, P.DTINATIVAUTO
                   FROM ( SELECT CASE
                                    WHEN PRATO.CDPRATONIVANTE IS NOT NULL
                                    THEN PRATO.CDPRATONIVANTE
                                    ELSE '0'
                                  END AS CDPRATONIVANTE,
                        PRATO.CDPRATO, PRATO.IDATIVO, PRATO.CDFILAUXPRAT, PRATO.CDNVPRATO, PRATO.NMPRATO, PRATO.CDFILIAL,
                        PRATO.SGUNIDADE, DECODE(UN.SGUNIDADE, NULL, '', UN.SGUNIDADE || '|' || UN.DSUNIDADE) AS DSUNIDADE, PRATO.NRFORMAPREPARO, FP.NMFORMAPREPARO, PRATO.NRORG, PRATO.VRARREDTRANS,
                        PRATO.QTTEMPOPRAT, PRATO.IDFREQUENCIA, PRATO.DTINATIVAUTO
                       FROM PRATO
                       LEFT JOIN UNIDMEDIDA UN ON UN.SGUNIDADE = PRATO.SGUNIDADE
                       LEFT JOIN FORMAPREPARO FP ON FP.NRFORMAPREPARO = PRATO.NRFORMAPREPARO
                            WHERE (CDFILIAL = :CDFILIAL OR PRATO.CDFILIAL IS NULL)
                        AND ( cdprato <> 'FFFFFFFFFF' AND cdprato <> 'PPPPPPPPPP')
                        AND $addWhere
                            ORDER BY CDPRATONIVANTE, CDPRATO
                      ) P
                    LEFT JOIN
                      (SELECT P.CDPRATO, COUNT(P2.CDPRATO) AS CHILDCOUNT
                      FROM PRATO P
                      LEFT JOIN PRATO P2 ON P.CDPRATO=P2.CDPRATONIVANTE
                      WHERE P2.CDFILAUXPRAT = :CDFILIAL OR P2.CDFILAUXPRAT = 'P'
                      GROUP BY P.CDPRATO
                      ) PCOUNT ON P.CDPRATO = PCOUNT.CDPRATO";

        $params = array(
            "CDFILIAL" => $filial
        );
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaPratosFilterAll($filial, $addWhere){
        $sql = "SELECT * FROM (SELECT
                    P.CDFILIAL,
                    P.IDATIVO,
                    P.CDNVPRATO,
                    CASE
                        WHEN P.CDPRATONIVANTE = '0'
                             AND P.CDNVPRATO = 1 THEN
                            NULL
                        ELSE
                            P.CDPRATONIVANTE
                    END AS CDPRATONIVANTE,
                    P.CDPRATO AS CDPRATO_AUX,
                    P.CDPRATO,
                    FORMATPRATMASK(P.CDPRATO) AS CDPRATOFORMATADO,
                    P.CDFILAUXPRAT,
                    P.NMPRATO,
                    DECODE(P.CDFILAUXPRAT, 'P', 'P', 'L') AS IDTIPO,
                    P.SGUNIDADE,
                    P.DSUNIDADE,
                    P.NRFORMAPREPARO,
                    P.NMFORMAPREPARO,
                    P.NRORG,
                    P.VRARREDTRANS,
                    P.QTTEMPOPRAT,
                    P.IDFREQUENCIA,
                    CASE
                        WHEN CHILDCOUNT > 0 THEN
                            'TRUE'
                        ELSE
                            'FALSE'
                    END AS HASCHILD,
                    P.DTINATIVAUTO
                FROM
                    (
                        SELECT
                            CASE
                                WHEN PRATO.CDPRATONIVANTE IS NOT NULL THEN
                                    PRATO.CDPRATONIVANTE
                                ELSE
                                    '0'
                            END AS CDPRATONIVANTE,
                            PRATO.CDPRATO,
                            PRATO.IDATIVO,
                            PRATO.CDFILAUXPRAT,
                            PRATO.CDNVPRATO,
                            PRATO.NMPRATO,
                            PRATO.CDFILIAL,
                            PRATO.SGUNIDADE,
                            DECODE(UN.SGUNIDADE, NULL, '', UN.SGUNIDADE
                                                           || '|'
                                                           || UN.DSUNIDADE) AS DSUNIDADE,
                            PRATO.NRFORMAPREPARO,
                            FP.NMFORMAPREPARO,
                            PRATO.NRORG,
                            PRATO.VRARREDTRANS,
                            PRATO.QTTEMPOPRAT,
                            PRATO.IDFREQUENCIA,
                            PRATO.DTINATIVAUTO
                        FROM
                            PRATO          LEFT
                            JOIN UNIDMEDIDA     UN ON UN.SGUNIDADE = PRATO.SGUNIDADE
                            LEFT JOIN FORMAPREPARO   FP ON FP.NRFORMAPREPARO = PRATO.NRFORMAPREPARO
                        WHERE
                            ( CDFILIAL = :CDFILIAL
                              OR PRATO.CDFILIAL IS NULL )
                            AND ( CDPRATO <> 'FFFFFFFFFF'
                                  AND CDPRATO <> 'PPPPPPPPPP' )
                        ORDER BY
                            CDPRATONIVANTE,
                            CDPRATO
                    ) P
                    INNER JOIN (
                        SELECT
                            P.CDPRATO,
                            COUNT(P2.CDPRATO) AS CHILDCOUNT
                        FROM
                            PRATO   P
                            LEFT JOIN PRATO   P2 ON P.CDPRATO = P2.CDPRATONIVANTE
                        GROUP BY
                            P.CDPRATO
                    ) PCOUNT ON P.CDPRATO = PCOUNT.CDPRATO)
                    WHERE ". $addWhere;

        $params = array(
            "CDFILIAL" => $filial
        );
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaPratosEpais($filial, $addWhere){
        $sql = "SELECT P.CDFILAUXPRAT, DECODE(P.CDFILAUXPRAT, 'P', 'P', 'L') AS IDTIPO,
                       P.CDPRATO, P.CDPRATO AS CDPRATO_AUX, P.CDFILIAL, P.NMPRATO, P.CDNVPRATO,
                       P.SGUNIDADE, UN.SGUNIDADE || '|' || UN.DSUNIDADE AS DSUNIDADE, P.VRARREDTRANS, P.DTULTATUPRAT, P.CDSEGMER,
                       P.CDEXPRATO, P.DTULTATURECE, P.CDPRODPRODUCAO, P.IDFREQUENCIA,
                       P.QTTEMPOPRAT, P.VRVALIDADE, P.NRFORMAPREPARO, FP.NRFORMAPREPARO || '|' || FP.NMFORMAPREPARO AS NMFORMAPREPARO, P.CDPRATONIVANTE,
                       'false' AS HASCHILD, P.DTINATIVAUTO
                  FROM (SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, NMPRATO, CDNVPRATO,
                               SGUNIDADE, VRARREDTRANS, DTULTATUPRAT, CDSEGMER,
                               CDEXPRATO, DTULTATURECE, CDPRODPRODUCAO, IDFREQUENCIA,
                               QTTEMPOPRAT, VRVALIDADE, NRFORMAPREPARO, CDPRATONIVANTE, DTINATIVAUTO
                          FROM PRATO
                       CONNECT BY PRIOR CDPRATONIVANTE = CDPRATO
                         START WITH CDPRATO in (SELECT PRATO.CDPRATONIVANTE
                                                  FROM PRATO
                                                 WHERE $addWhere)
                         UNION

                        SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, NMPRATO, CDNVPRATO,
                               SGUNIDADE, VRARREDTRANS, DTULTATUPRAT, CDSEGMER,
                               CDEXPRATO, DTULTATURECE, CDPRODPRODUCAO, IDFREQUENCIA,
                               QTTEMPOPRAT, VRVALIDADE, NRFORMAPREPARO, CDPRATONIVANTE, DTINATIVAUTO
                          FROM PRATO
                         WHERE $addWhere
                         ORDER BY CDPRATO) P
                  LEFT JOIN UNIDMEDIDA UN ON UN.SGUNIDADE = P.SGUNIDADE
                  LEFT JOIN FORMAPREPARO FP ON FP.NRFORMAPREPARO = P.NRFORMAPREPARO
                 WHERE (P.CDFILIAL = :CDFILIAL OR P.CDFILIAL IS NULL)";

        $params = array(
            "CDFILIAL" => $filial
        );
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaFilhos($cdPrato, $filAuxPai) {
        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Prato')
            ->findBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                'cdpratonivante' => $cdPrato,
                'cdfilauxprat' => $filAuxPai),
                array('cdprato' => 'ASC'));
    }

    public function retornaFilhosfiliauxpratMaisFilial($cdPrato,$filAuxPai,$filial) {
        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Prato')
            ->findBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                'cdpratonivante' => $cdPrato,
                'cdfilauxprat' => [$filAuxPai,$filial]),
                array('cdprato' => 'ASC'));
    }

    public function retornaPrato($cdPrato, $filAuxPai) {
        $sql = "SELECT * FROM PRATO 
                 WHERE CDFILAUXPRAT IN (:CDFILAUXPRAT, 'P') 
                   AND CDPRATO =:CDPRATO 
                   AND NRORG = :NRORG 
                 ORDER BY CDPRATO DESC";

        $params = array(
            "CDPRATO"=>$cdPrato,
            "CDFILAUXPRAT" => $filAuxPai,
            "NRORG" => $this->environment->getNrOrgTrab()
        );
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function excluiPrato($cdPrato, $filAuxPai){
        try {
            $prato = $this->retornaPratoExclusao($cdPrato, $filAuxPai);
            $this->entityManager->remove($prato);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaPratoExclusao($cdPrato, $filAuxPai) {
        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Prato')
            ->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                'cdprato' => $cdPrato,
                'cdfilauxprat' => ['P', $filAuxPai]),
                array('cdprato' => 'DESC'));
    }

    public function excluiNivelVazio($cdPrato, $filAuxPai){
        $sql = "DELETE FROM PRATO WHERE CDFILAUXPRAT =:CDFILAUXPRAT AND CDPRATO =:CDPRATO";

        $params = array(
            "CDPRATO"=>$cdPrato,
            "CDFILAUXPRAT" => $filAuxPai
        );
        try {
            
            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            // return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function excluiContainerPrato($cdPrato, $cdfilauxprat){
        try {
            $params = array(
                'cdprato' => $cdPrato,
                'cdfilauxprat' => $cdfilauxprat,
                'nrorg' => $this->environment->getNrOrgTrab()
            );

            $pratosContainer = $this->entityManager->getRepository(
                '\Teknisa\Model\Planejamento\Entities\Pratocontainer'
            )->findBy($params);

            if (isset($pratosContainer)) {
                foreach ($pratosContainer as $prato) {
                    $this->entityManager->remove($prato);
                    $this->entityManager->flush();
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

/*
    public function atualizaEmbalagemPrincipal($cdPrato, $filAuxPai) {
        $pratoContainers = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Pratocontainer')
            ->findBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                'cdprato' => $cdPrato,
                'cdfilauxprat' => $filAuxPai));

        if (isset($pratosContainer)) {
            foreach ($pratoContainers as $associacao) {
                $associacao->setIdpratcontpri('N');
                $this->entityManager->flush();
            }
        }

    }
*/


    public function retornaPreparoPadrao($prato, $filAux, $query = null){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO,
                       NRPREPPRAT, IDPREPPRAT, TXPREPPRAT, NMRPREPPRAT, NRORG,
                       DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO,
                       NRORGULTATU, CDOPERULTATU, IDATIVO
                  FROM PREPPRAPAD
                 WHERE CDPRATO = :CDPRATO 
                   AND CDFILAUXPRAT = :CDFILAUXPRAT ". $query."
                 ORDER BY NRPREPPRAT DESC";

        $params = array(
            "CDPRATO" => $prato,
            "CDFILAUXPRAT" => $filAux
        );
        
        try {
            $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
            if ($modos) {
                foreach ($modos as $i => $modo) {
                    $preparo = str_replace('\pard', '<br>', $modos[$i]['TXPREPPRAT']);
                    $preparo = str_replace('\par', '<br>', $preparo);
                    $modos[$i]['TXPREPPRAT'] = DataSourceUtil::toPlainText($preparo);
                }
            }

            return $modos;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function inserePreparoPadrao(
        $filAux, $prato, $codigo, $status, $texto,
        $reponsavel, $data, $nrorg, $operador
    ) {
        try {
            $insert = "INSERT INTO PREPPRAPAD (
                         CDFILAUXPRAT, CDPRATO, NRPREPPRAT, IDPREPPRAT, TXPREPPRAT,
                         NMRPREPPRAT, NRORG, DTINCLUSAO, DTULTATU, NRORGINCLUSAO,
                         CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO)
                       VALUES (
                         :CDFILAUXPRAT, :CDPRATO, :NRPREPPRAT, :IDPREPPRAT, :TXPREPPRAT,
                         :NMRPREPPRAT, :NRORG, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO,
                         :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU, :IDATIVO)";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "NRPREPPRAT"=> $codigo,
                "IDPREPPRAT"=> $status,
                "NMRPREPPRAT"=> $reponsavel,
                "NRORG"=> $nrorg,
                "DTINCLUSAO"=> $data,
                "DTULTATU"=> $data,
                "NRORGINCLUSAO"=> $nrorg,
                "CDOPERINCLUSAO"=> $operador,
                "NRORGULTATU"=> $nrorg,
                "CDOPERULTATU"=> $operador,
                "IDATIVO" => 'S'
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
//            $texto = utf8_encode($texto);
            $stmt->bindParam('TXPREPPRAT', $texto, SQLT_LBI);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function atualizaPreparoPadrao(
        $filAux, $prato, $codigo, $status, $texto,
        $reponsavel, $data, $nrorg, $operador
    ) {
        try {
            $update = "UPDATE PREPPRAPAD
                          SET IDPREPPRAT = :IDPREPPRAT,
                              NMRPREPPRAT = :NMRPREPPRAT,
                              TXPREPPRAT = :TXPREPPRAT,
                              DTULTATU = :DTULTATU,
                              NRORGULTATU = :NRORGULTATU,
                              CDOPERULTATU = :CDOPERULTATU
                        WHERE CDFILAUXPRAT = :CDFILAUXPRAT
                          AND CDPRATO = :CDPRATO
                          AND NRPREPPRAT = :NRPREPPRAT";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "NRPREPPRAT"=> $codigo,
                "IDPREPPRAT"=> 1,
                "NMRPREPPRAT"=> $reponsavel,
                "DTULTATU"=> $data,
                "NRORGULTATU"=> $nrorg,
                "CDOPERULTATU"=> $operador
            );

            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->bindParam('TXPREPPRAT', $texto, SQLT_LBI);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function atualizaStatusPreparoPadrao(
        $filAux, $prato
    ) {
        try {
            $update = "UPDATE PREPPRAPAD
                      SET IDPREPPRAT = :IDPREPPRAT
                    WHERE CDFILAUXPRAT = :CDFILAUXPRAT
                      AND CDPRATO = :CDPRATO";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "IDPREPPRAT"=> '2'
            );

            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaPreparoPadraoQualidade($prato, $filAux){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO,
                       NRPREPPADQ, IDPREPPADQ, TXPREPPADQ, NMPREPPADQ, NRORG,
                       DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO,
                       NRORGULTATU, CDOPERULTATU, IDATIVO
                  FROM PREPPRAPADQUALI
                 WHERE CDPRATO = :CDPRATO
                   AND CDFILAUXPRAT = :CDFILAUXPRAT
                 ORDER BY NRPREPPADQ DESC";

        $params = array(
            "CDPRATO" => $prato,
            "CDFILAUXPRAT" => $filAux
        );

        try {
            $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
            if ($modos) {
                foreach ($modos as $i => $modo) {
                    $preparo = str_replace('\pard', '<br>', $modos[$i]['TXPREPPADQ']);
                    $preparo = str_replace('\par', '<br>', $preparo);
                    $modos[$i]['TXPREPPADQ'] = DataSourceUtil::toPlainText($preparo);
                }
            }

            return $modos;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function inserePreparoPadraoQualidade(
        $filAux, $prato, $codigo, $status, $texto,
        $reponsavel, $data, $nrorg, $operador, $newCode, $nrpad
    ) {
        try {
            $insert = "INSERT INTO PREPPRAPADQUALI (
                         CDFILAUXPRAT, CDPRATO, NRPREPPADQ, IDPREPPADQ, TXPREPPADQ,
                         NMPREPPADQ, NRPREPPRAPADQUALI, NRORG, DTINCLUSAO, DTULTATU, NRORGINCLUSAO,
                         CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRPADRAOQUALIDADE)
                       VALUES (
                         :CDFILAUXPRAT, :CDPRATO, :NRPREPPADQ, :IDPREPPADQ, :TXPREPPADQ,
                         :NMPREPPADQ, :NRPREPPRAPADQUALI, :NRORG, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO,
                         :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU, :IDATIVO, :NRPADRAOQUALIDADE)";

            $params = array(
                "CDFILAUXPRAT"     => $filAux,
                "CDPRATO"          => $prato,
                "NRPREPPADQ"       => $codigo,
                "IDPREPPADQ"       => $status,
                "NMPREPPADQ"       => $reponsavel,
                "NRORG"            => $nrorg,
                "DTINCLUSAO"       => $data,
                "DTULTATU"         => $data,
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERINCLUSAO"   => $operador,
                "NRORGULTATU"      => $nrorg,
                "CDOPERULTATU"     => $operador,
                "IDATIVO"          => 'S',
                "NRPREPPRAPADQUALI"=> $newCode,
                "NRPADRAOQUALIDADE"=> $nrpad,
                'TXPREPPADQ'       => $texto
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaPreparoPadraoQualidade(
        $status, $texto, $reponsavel, $data, $nrorg, 
        $operador, $id, $codigo
    ) {
        try {
            $update = "UPDATE PREPPRAPADQUALI
                          SET IDPREPPADQ = :IDPREPPADQ,
                              NMPREPPADQ = :NMPREPPADQ,
                              NRPREPPADQ = :NRPREPPADQ,
                              TXPREPPADQ = :TXPREPPADQ,
                              DTULTATU = :DTULTATU,
                              NRORGULTATU = :NRORGULTATU,
                              CDOPERULTATU = :CDOPERULTATU
                        WHERE NRPREPPRAPADQUALI = :NRPREPPRAPADQUALI";

            $params = array(
                "IDPREPPADQ"       => $status,
                "NMPREPPADQ"       => $reponsavel,
                "NRPREPPADQ"       => $codigo,
                'TXPREPPADQ'       => $texto,
                "DTULTATU"         => $data,
                "NRORGULTATU"      => $nrorg,
                "CDOPERULTATU"     => $operador,
                "NRPREPPRAPADQUALI"=> $id
            );
            
            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaPreparoRegiao($prato, $filAux){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO,
                       NRPREPREGIAO, IDPREPREGIAO, TXPREPREGIAO, NMPREPREGIAO, NRORG,
                       DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO,
                       NRORGULTATU, CDOPERULTATU, IDATIVO
                  FROM PREPPRAREGIAO
                 WHERE CDPRATO = :CDPRATO
                   AND CDFILAUXPRAT = :CDFILAUXPRAT
                 ORDER BY NRPREPREGIAO DESC";

        $params = array(
            "CDPRATO" => $prato,
            "CDFILAUXPRAT" => $filAux
        );

        try {
            $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
            if ($modos) {
                foreach ($modos as $i => $modo) {
                    $preparo = str_replace('\pard', '<br>', $modos[$i]['TXPREPREGIAO']);
                    $preparo = str_replace('\par', '<br>', $preparo);
                    $modos[$i]['TXPREPREGIAO'] = DataSourceUtil::toPlainText($preparo);
                }
            }

            return $modos;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaPreparoRegiao(
        $status, $texto, $reponsavel, $data, $nrorg, 
        $operador, $regiao, $id, $codigo
    ) {
        try {
            $update = "UPDATE PREPPRAREGIAO
                          SET IDPREPREGIAO = :IDPREPREGIAO,
                              NMPREPREGIAO = :NMPREPREGIAO,
                              NRPREPREGIAO = :NRPREPREGIAO,
                              TXPREPREGIAO = :TXPREPREGIAO,
                              DTULTATU = :DTULTATU,
                              NRORGULTATU = :NRORGULTATU,
                              CDOPERULTATU = :CDOPERULTATU
                        WHERE NRPREPPRAREGIAO = :NRPREPPRAREGIAO";

            $params = array(
                
                "IDPREPREGIAO"   => $status,
                "NMPREPREGIAO"   => $reponsavel,
                "NRPREPREGIAO"   => $codigo,
                'TXPREPREGIAO'   => $texto,
                "DTULTATU"       => $data,
                "NRORGULTATU"    => $nrorg,
                "CDOPERULTATU"   => $operador,
                "NRPREPPRAREGIAO"=> $id
            );
            
            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function inserePreparoRegiao(
        $filAux, $prato, $codigo, $status, $texto,
        $reponsavel, $data, $nrorg, $operador, $newCode, $regiao, $nrpad
    ) {
        try {
            $insert = "INSERT INTO PREPPRAREGIAO (
                         CDFILAUXPRAT, CDPRATO, NRPREPREGIAO, IDPREPREGIAO, TXPREPREGIAO,
                         NMPREPREGIAO, NRPREPPRAREGIAO, CDREGIAO, NRORG, DTINCLUSAO, DTULTATU, NRORGINCLUSAO,
                         CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO, NRPADRAOQUALIDADE)
                       VALUES (
                         :CDFILAUXPRAT, :CDPRATO, :NRPREPREGIAO, :IDPREPREGIAO, :TXPREPREGIAO,
                         :NMPREPREGIAO, :NRPREPPRAREGIAO, :CDREGIAO, :NRORG, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO,
                         :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU, :IDATIVO, :NRPADRAOQUALIDADE)";

            $params = array(
                "CDFILAUXPRAT"     => $filAux,
                "CDPRATO"          => $prato,
                "NRPREPREGIAO"     => $codigo,
                "IDPREPREGIAO"     => $status,
                "NMPREPREGIAO"     => $reponsavel,
                "CDREGIAO"         => $regiao,
                "NRORG"            => $nrorg,
                "DTINCLUSAO"       => $data,
                "DTULTATU"         => $data,
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERINCLUSAO"   => $operador,
                "NRORGULTATU"      => $nrorg,
                "CDOPERULTATU"     => $operador,
                "IDATIVO"          => 'S',
                "NRPREPPRAREGIAO"  => $newCode,
                "NRPADRAOQUALIDADE"=> $nrpad,
                'TXPREPREGIAO'     => $texto
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function retornaPreparoLocal($prato, $filAux, $filial, $query = null){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL,
                       NRPREPFILI, IDPREPFILI, TXPREPFILI, NMRPREPFILI, NRORG,
                       DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO,
                       NRORGULTATU, CDOPERULTATU, IDATIVO
                  FROM PREPPRAFIL
                 WHERE CDPRATO = :CDPRATO
                   AND CDFILAUXPRAT = :CDFILAUXPRAT
                   AND CDFILIAL = :CDFILIAL ". $query."
                 ORDER BY NRPREPFILI DESC";

        $params = array(
            "CDPRATO" => $prato,
            "CDFILAUXPRAT" => $filAux,
            "CDFILIAL" => $filial
        );

        try {
            $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
            if ($modos) {
                foreach ($modos as $i => $modo) {
                    $preparo = str_replace('\pard', '<br>', $modos[$i]['TXPREPFILI']);
                    $preparo = str_replace('\par', '<br>', $preparo);
                    $modos[$i]['TXPREPFILI'] = DataSourceUtil::toPlainText($preparo);
                }
            }

            return $modos;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function inserePreparoLocal(
        $filAux, $prato, $filial, $codigo, $status, $texto,
        $reponsavel, $data, $nrorg, $operador
    ) {
        try {
            $insert = "INSERT INTO PREPPRAFIL (
                         CDFILAUXPRAT, CDPRATO, CDFILIAL,
                         NRPREPFILI, IDPREPFILI, TXPREPFILI, NMRPREPFILI, NRORG,
                         DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO,
                         NRORGULTATU, CDOPERULTATU, IDATIVO)
                       VALUES (
                         :CDFILAUXPRAT, :CDPRATO, :CDFILIAL,
                         :NRPREPFILI, :IDPREPFILI, :TXPREPFILI, :NMRPREPFILI, :NRORG,
                         :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO,
                         :NRORGULTATU, :CDOPERULTATU, :IDATIVO)";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "CDFILIAL"=> $filial,
                "NRPREPFILI"=> $codigo,
                "IDPREPFILI"=> $status,
                "NMRPREPFILI"=> $reponsavel,
                "NRORG"=> $nrorg,
                "DTINCLUSAO"=> $data,
                "DTULTATU"=> $data,
                "NRORGINCLUSAO"=> $nrorg,
                "CDOPERINCLUSAO"=> $operador,
                "NRORGULTATU"=> $nrorg,
                "CDOPERULTATU"=> $operador,
                "IDATIVO" => 'S'
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->bindParam('TXPREPFILI', $texto, SQLT_LBI);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function atualizaPreparoLocal(
        $filAux, $prato, $filial, $codigo, $status, $texto,
        $reponsavel, $data, $nrorg, $operador
    ) {
        try {
            $update = "UPDATE PREPPRAFIL
                          SET IDPREPFILI = :IDPREPFILI,
                              NMRPREPFILI = :NMRPREPFILI,
                              TXPREPFILI = :TXPREPFILI,
                              DTULTATU = :DTULTATU,
                              NRORGULTATU = :NRORGULTATU,
                              CDOPERULTATU = :CDOPERULTATU
                        WHERE CDFILAUXPRAT = :CDFILAUXPRAT
                          AND CDPRATO = :CDPRATO
                          AND CDFILIAL = :CDFILIAL
                          AND NRPREPFILI = :NRPREPFILI";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "CDFILIAL"=> $filial,
                "NRPREPFILI"=> $codigo,
                "IDPREPFILI"=> $status,
                "NMRPREPFILI"=> $reponsavel,
                "DTULTATU"=> $data,
                "NRORGULTATU"=> $nrorg,
                "CDOPERULTATU"=> $operador
            );

            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->bindParam('TXPREPFILI', $texto, SQLT_LBI);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function atualizaStatusPreparoLocal(
        $filAux, $prato, $filial
    ) {
        try {
            $update = "UPDATE PREPPRAFIL
                      SET IDPREPFILI = :IDPREPFILI
                    WHERE CDFILAUXPRAT = :CDFILAUXPRAT
                      AND CDPRATO = :CDPRATO
                      AND CDFILIAL = :CDFILIAL";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "CDFILIAL"=> $filial,
                "IDPREPFILI"=> '2'
            );

            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function retornaPreparoServico($prato, $filAux, $filial, $servico, $query = null){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, CDSERVFILI,
                       NRPREPSERV, IDPREPSERV, TXPREPSERV, NMRPREPSERV, NRORG,
                       DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO,
                       NRORGULTATU, CDOPERULTATU, IDATIVO
                  FROM PREPPRASER
                 WHERE CDPRATO = :CDPRATO
                   AND CDFILAUXPRAT = :CDFILAUXPRAT
                   AND CDFILIAL = :CDFILIAL
                   AND CDSERVFILI = :CDSERVFILI ".$query."
                 ORDER BY NRPREPSERV DESC";

        $params = array(
            "CDPRATO" => $prato,
            "CDFILAUXPRAT" => $filAux,
            "CDFILIAL" => $filial,
            "CDSERVFILI" => $servico
        );

        try {
            $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
            if ($modos) {
                foreach ($modos as $i => $modo) {
                    $preparo = str_replace('\pard', '<br>', $modos[$i]['TXPREPSERV']);
                    $preparo = str_replace('\par', '<br>', $preparo);
                    $modos[$i]['TXPREPPRAT'] = DataSourceUtil::toPlainText($preparo);
                }
            }

            return $modos;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function inserePreparoServico(
        $filAux, $prato, $filial, $servico, $codigo, $status, $texto,
        $reponsavel, $data, $nrorg, $operador
    ) {
        try {
            $insert = "INSERT INTO PREPPRASER (
                         CDFILAUXPRAT, CDPRATO, CDFILIAL, CDSERVFILI,
                       NRPREPSERV, IDPREPSERV, TXPREPSERV, NMRPREPSERV, NRORG,
                         DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO,
                         NRORGULTATU, CDOPERULTATU, IDATIVO)
                       VALUES (
                         :CDFILAUXPRAT, :CDPRATO, :CDFILIAL, :CDSERVFILI,
                         :NRPREPSERV, :IDPREPSERV, :TXPREPSERV, :NMRPREPSERV, :NRORG,
                         :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO,
                         :NRORGULTATU, :CDOPERULTATU, :IDATIVO)";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "CDFILIAL"=> $filial,
                "CDSERVFILI"=> $servico,
                "NRPREPSERV"=> $codigo,
                "IDPREPSERV"=> $status,
                "NMRPREPSERV"=> $reponsavel,
                "NRORG"=> $nrorg,
                "DTINCLUSAO"=> $data,
                "DTULTATU"=> $data,
                "NRORGINCLUSAO"=> $nrorg,
                "CDOPERINCLUSAO"=> $operador,
                "NRORGULTATU"=> $nrorg,
                "CDOPERULTATU"=> $operador,
                "IDATIVO" => 'S'
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->bindParam('TXPREPSERV', $texto, SQLT_LBI);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function atualizaPreparoServico(
        $filAux, $prato, $filial, $servico, $codigo, $status, $texto,
        $reponsavel, $data, $nrorg, $operador
    ) {
        try {
            $update = "UPDATE PREPPRASER
                          SET IDPREPSERV = :IDPREPSERV,
                              NMRPREPSERV = :NMRPREPSERV,
                              TXPREPSERV = :TXPREPSERV,
                              DTULTATU = :DTULTATU,
                              NRORGULTATU = :NRORGULTATU,
                              CDOPERULTATU = :CDOPERULTATU
                        WHERE CDFILAUXPRAT = :CDFILAUXPRAT
                          AND CDPRATO = :CDPRATO
                          AND CDFILIAL = :CDFILIAL
                          AND CDSERVFILI = :CDSERVFILI
                          AND NRPREPSERV = :NRPREPSERV";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "CDFILIAL"=> $filial,
                "CDSERVFILI"=> $servico,
                "NRPREPSERV"=> $codigo,
                "IDPREPSERV"=> $status,
                "NMRPREPSERV"=> $reponsavel,
                "DTULTATU"=> $data,
                "NRORGULTATU"=> $nrorg,
                "CDOPERULTATU"=> $operador
            );

            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->bindParam('TXPREPSERV', $texto, SQLT_LBI);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function atualizaStatusPreparoServico(
        $filAux, $prato, $filial, $servico
    ) {
        try {
            $update = "UPDATE PREPPRASER
                      SET IDPREPSERV = :IDPREPSERV
                    WHERE CDFILAUXPRAT = :CDFILAUXPRAT
                      AND CDPRATO = :CDPRATO
                      AND CDFILIAL = :CDFILIAL
                      AND CDSERVFILI = :CDSERVFILI";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "CDFILIAL"=> $filial,
                "CDSERVFILI"=> $servico,
                "IDPREPSERV"=> '2'
            );

            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function retornaFatorConversao($produto, $sgunidade, $nrorg){
        $sql = "SELECT DISTINCT SGUNIDADE, VRFATOCONVES, CDPRODUTO, NRORG
                  FROM (SELECT SGUNIDADE, VRFATOCONVES, CDPRODUTO, NRORG
                          FROM CONVPRODUTO
                         WHERE CDPRODUTO = :CDPRODUTO
                           AND NRORG       = :NRORG
                         UNION ALL
                        SELECT SGUNIDDE         AS SGUNIDADE,
                               TO_NUMBER(VRFTCONVER) AS VRFATOCONVES,
                               :CDPRODUTO            AS CDPRODUTO,
                               NRORG
                          FROM CONVUNID
                         WHERE SGUNIDDE = :SGUNIDADE
                           AND NRORG        = :NRORG
                         UNION ALL
                        SELECT :SGUNIDADE          AS SGUNIDADE,
                               TO_NUMBER(1) AS VRFATOCONVES,
                               :CDPRODUTO               AS CDPRODUTO,
                               TO_NUMBER(:NRORG)        AS NRORG
                          FROM PARAMGERAL
                      ) CONVPRODUTO";

        $params = array(
            "CDPRODUTO" => $produto,
            "SGUNIDADE" => $sgunidade,
            "NRORG" => $nrorg
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaReceitaPadrao($cdprato, $cdFilAuxPrat, $produto = null){
        if ($produto) {
            $produtos = $this->entityManager->getRepository('Teknisa\Model\Planejamento\Entities\Recepadr')
                ->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                    'cdprato' => $cdprato,
                    'cdfilauxprat' => $cdFilAuxPrat,
                    'cdproduto' => $produto));
        }
        else {
            $produtos = $this->entityManager->getRepository('Teknisa\Model\Planejamento\Entities\Recepadr')
                ->findBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                    'cdprato' => $cdprato,
                    'cdfilauxprat' => $cdFilAuxPrat));
        }
        if (isset($produtos)) {
            return $produtos;
        } else {
            return false;
        }
    }

    public function saveRecefiliserv($produto) {
        $this->entityManager->persist($produto);
    }

    public function retornaFatorConversaoPrato($produto, $sgunidade, $sgunidadeCompar){
         $sql = "SELECT TO_NUMBER(C.VRFATOCONVES) AS VRFATOCONVES,
                        RPAD('/', 1, ' ')       AS IDTIPOPE,
                        0 AS ORDEM
                   FROM CONVPRODUTO C
                   LEFT JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODUTO
                  WHERE (C.CDPRODUTO = :P_CDPRODUTO)
                    AND (P.SGUNIDADE = :P_SGUNIDADEORI)
                    AND (C.SGUNIDADE = :P_SGUNIDADEDEST)

                  UNION ALL

                 SELECT TO_NUMBER(C.VRFATOCONVES) AS VRFATOCONVES,
                        RPAD('*', 1, ' ')       AS IDTIPOPE,
                        1 AS ORDEM
                   FROM CONVPRODUTO C
                   LEFT JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODUTO
                  WHERE (C.CDPRODUTO = :P_CDPRODUTO)
                    AND (P.SGUNIDADE = :P_SGUNIDADEDEST)
                    AND (C.SGUNIDADE = :P_SGUNIDADEORI)

                  UNION ALL

                 SELECT TO_NUMBER(VRFTCONVER)  AS VRFATOCONVES,
                        RPAD(IDTIPOPE, 1, ' ') AS IDTIPOPE,
                        2 AS ORDEM
                   FROM CONVUNID
                  WHERE (SGUNIDPARA = :P_SGUNIDADEDEST)
                    AND (SGUNIDDE = :P_SGUNIDADEORI)

                  UNION ALL

                 SELECT TO_NUMBER(VRFTCONVER)  AS VRFATOCONVES,
                        RPAD(IDTIPOPE, 1, ' ') AS IDTIPOPE,
                        3 AS ORDEM
                   FROM CONVUNID
                  WHERE (SGUNIDPARA = :P_SGUNIDADEORI )
                    AND (SGUNIDDE = :P_SGUNIDADEDEST )

                  ORDER BY ORDEM";

        $params = array(
            "P_CDPRODUTO" => $produto,
            "P_SGUNIDADEORI" => $sgunidade,
            "P_SGUNIDADEDEST" => $sgunidadeCompar
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaReceitaLocal($cdprato, $cdFilAuxPrat, $filial, $produto = null) {
        if ($produto) {
            $produtos = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Recefili')
                ->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                    'cdprato' => $cdprato,
                    'cdfilauxprat' => $cdFilAuxPrat,
                    'cdfilial' => $filial,
                    'cdproduto' => $produto));
        }
        else {
            $produtos = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Recefili')
                ->findBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                    'cdprato' => $cdprato,
                    'cdfilauxprat' => $cdFilAuxPrat,
                    'cdfilial' => $filial),
                    array('cdproduto' => 'ASC'));
        }

        if (isset($produtos)) {
            return $produtos;
        } else {
            return false;
        }
    }

    public function retornaReceitaServico($cdprato, $cdFilAuxPrat, $filial, $servico, $produto = null){
        if ($produto) {
             
            $produtos = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Receserv')
                ->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                    'cdprato' => $cdprato,
                    'cdfilauxprat' => $cdFilAuxPrat,
                    'cdfilial' => $filial,
                    'cdservfili' => $servico,
                    'cdproduto' => $produto));
        } else {
            $produtos = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Receserv')
                ->findBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                    'cdprato' => $cdprato,
                    'cdfilauxprat' => $cdFilAuxPrat,
                    'cdfilial' => $filial,
                    'cdservfili' => $servico),
                    array('cdproduto' => 'ASC'));
        }

        if (isset($produtos)) {
            return $produtos;
        } else {
            return false;
        }
    }

    public function excluiReceita($produtoReceita){
        try {
            if (isset($produtoReceita)) {
                $this->entityManager->remove($produtoReceita);
                $this->entityManager->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function iniciaTransacao(){
        $this->entityManager->beginTransaction();
    }

    public function getConnection(){
        return $this->entityManager->getConnection();
    }

    public function flush(){
        $this->entityManager->flush();
    }

    public function commit(){
        $this->entityManager->commit();
    }

    public function rollback(){
        $this->entityManager->rollback();
    }

    public function safeRollback(){
        if($this->entityManager->getConnection()->isTransactionActive()){
            $this->entityManager->rollback();
        }
    }


    public function retornaReceitaPrato($prato, $filauxprat) {
        $sql = "SELECT REC.CDPRATO, REC.CDFILAUXPRAT
                  FROM (SELECT CDPRATO, CDFILAUXPRAT
                          FROM RECEPADR
                         WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT
                         UNION ALL
                        SELECT CDPRATO, CDFILAUXPRAT
                          FROM RECEPADR
                         WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT
                         UNION ALL
                        SELECT CDPRATO, CDFILAUXPRAT
                          FROM RECEPADR
                         WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT
                       ) REC";

        $params = array(
            "CDPRATO" => $prato,
            "CDFILAUXPRAT" => $filauxprat
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaCorSinaSafra($valor){
        $sql = "SELECT VRLIMINFSAFR, VRLIMSUPSAFR, NMSINASAFRA,
                       NRCORSINSAFR, DTULTATUSISA
                  FROM SINASAFRA
                 WHERE VRLIMINFSAFR <  :P_PERCSAFRA
                   AND VRLIMSUPSAFR >= :P_PERCSAFRA
                    OR :P_PERCSAFRA =  0
                   AND VRLIMINFSAFR =  :P_PERCSAFRA";

        $params = array(
            "P_PERCSAFRA" => $valor
        );

        try {
            $cor = $this->entityManager->getConnection()->fetchAll($sql, $params);
            if ($cor) {
                return $cor[0]['NRCORSINSAFR'];
            } else {
                return '';
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaTipoReceita($cdPrato, $cdFiliAuxPrat, $receita, $filial, $servico) { 
        $cdProdProducao;
        $sqlReceitaPadrao = "SELECT CDPRODPRODUCAO FROM PRATPADR WHERE CDFILAUXPRAT = :P_CDFILAUXPRAT AND CDPRATO = :P_CDPRATO";
                                
        $sqlReceitaLocal = "SELECT CDPRODPRODUCAO FROM PRATFILI WHERE CDFILAUXPRAT = :P_CDFILAUXPRAT AND CDPRATO = :P_CDPRATO AND CDFILIAL = :P_CDFILIAL";
                                
        $sqlReceitaServico = "SELECT CDPRODPRODUCAO FROM PRATSERV WHERE CDFILAUXPRAT = :P_CDFILAUXPRAT AND CDPRATO = :P_CDPRATO AND CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI";
    
        $sqlBloqueiaReceita = "SELECT COUNT(*) FROM (SELECT 0 AS ID, 'Padrão' AS TIPO, '' AS CDFILIAL, '' AS CDSERVFILI FROM PRATPADR WHERE CDPRODPRODUCAO = :P_CDPRODPRODUCAO AND ((:RECEITA <> 'P') OR (:RECEITA = 'P' AND CDPRATO||CDFILAUXPRAT <> :P_CDPRATO||:P_CDFILAUXPRAT)) UNION ALL SELECT 1 AS ID, 'Filial' AS TIPO, CDFILIAL, '' AS CDSERVFILI FROM PRATFILI WHERE CDPRODPRODUCAO = :P_CDPRODPRODUCAO AND ((:RECEITA <> 'L') OR (:RECEITA = 'L' AND (CDFILIAL <> :P_CDFILIAL OR CDPRATO||CDFILAUXPRAT <> :P_CDPRATO||:P_CDFILAUXPRAT))) UNION ALL SELECT 2 AS ID, 'Serviço' AS TIPO, CDFILIAL, CDSERVFILI FROM PRATSERV WHERE CDPRODPRODUCAO = :P_CDPRODPRODUCAO AND ((:RECEITA <> 'S') OR (:RECEITA = 'S' AND (CDFILIAL||CDSERVFILI <> :P_CDFILIAL||:P_CDSERVFILI OR CDPRATO||CDFILAUXPRAT <> :P_CDPRATO||:P_CDFILAUXPRAT))))"; 
        
        $paramsPadrao = array(
            "P_CDFILAUXPRAT" => $cdFiliAuxPrat,
            "P_CDPRATO" => $cdPrato
        );
        $paramsLocal = array(
            "P_CDFILAUXPRAT" => $cdFiliAuxPrat,
            "P_CDPRATO" => $cdPrato,
            "P_CDFILIAL" => $filial
        );
        $paramsServico = array(
            "P_CDFILAUXPRAT" => $cdFiliAuxPrat,
            "P_CDPRATO" => $cdPrato,
            "P_CDFILIAL" => $filial, 
            "P_CDSERVFILI" => $servico
        );
        $paramsBloqueiaReceita = array (
            "P_CDPRODPRODUCAO" => null,
            "RECEITA" => $receita,
            "P_CDPRATO" => $cdPrato,
            "P_CDFILAUXPRAT" => $cdFiliAuxPrat,
            "P_CDFILIAL" => $filial,
            "P_CDSERVFILI" => $servico
        );
        
        try{
            if($receita == "P"){ 
                $stmt = $this->entityManager->getConnection()->fetchAll($sqlReceitaPadrao, $paramsPadrao);
                if($stmt) {
                    $paramsBloqueiaReceita["P_CDPRODPRODUCAO"] = $stmt[0]["CDPRODPRODUCAO"];
                }
                $stmtQueryBloqueio = $this->entityManager->getConnection()->fetchAll($sqlBloqueiaReceita, $paramsBloqueiaReceita);
                return $stmtQueryBloqueio[0]["COUNT(*)"];
            }
            else if($receita == "L"){ 
                $stmt = $this->entityManager->getConnection()->fetchAll($sqlReceitaLocal, $paramsLocal);
                if($stmt) {
                    $paramsBloqueiaReceita["P_CDPRODPRODUCAO"] = $stmt[0]["CDPRODPRODUCAO"];
                }
                $stmtQueryBloqueio = $this->entityManager->getConnection()->fetchAll($sqlBloqueiaReceita, $paramsBloqueiaReceita);
                return $stmtQueryBloqueio[0]["COUNT(*)"];
            }
            else if($receita == "S"){ 
                $stmt = $this->entityManager->getConnection()->fetchAll($sqlReceitaServico, $paramsServico);
                if($stmt) {
                    $paramsBloqueiaReceita["P_CDPRODPRODUCAO"] = $stmt[0]["CDPRODPRODUCAO"];
                }
                $stmtQueryBloqueio = $this->entityManager->getConnection()->fetchAll($sqlBloqueiaReceita, $paramsBloqueiaReceita); 
                return $stmtQueryBloqueio[0]["COUNT(*)"];
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function retornaPratPadr($cdPrato, $cdfilauxprat){
        $params = array(
            'cdprato' => $cdPrato,
            'cdfilauxprat' => $cdfilauxprat,
            'nrorg' => $this->environment->getNrOrgTrab()
        );

        $pratPadr = $this->entityManager->getRepository(
            '\Teknisa\Model\Planejamento\Entities\Pratpadr'
        )->findOneBy($params);

        return $pratPadr;
    }

    public function retornaPratFili($cdPrato, $cdfilauxprat, $filial){
        $params = array(
            'cdprato' => $cdPrato,
            'cdfilauxprat' => $cdfilauxprat,
            'cdfilial' => $filial,
            'nrorg' => $this->environment->getNrOrgTrab()
        );

        $pratFili = $this->entityManager->getRepository(
            '\Teknisa\Model\Planejamento\Entities\Pratfili'
        )->findOneBy($params);

        return $pratFili;
    }

    public function retornaPratServ($cdPrato, $cdfilauxprat, $filial, $servico){
        $params = array(
            'cdprato' => $cdPrato,
            'cdfilauxprat' => $cdfilauxprat,
            'cdfilial' => $filial,
            'cdservfili' => $servico,
            'nrorg' => $this->environment->getNrOrgTrab()
        );

        $pratServ = $this->entityManager->getRepository(
            '\Teknisa\Model\Planejamento\Entities\Pratserv'
        )->findOneBy($params);

        return $pratServ;
    }

    public function excluiDadosReceita($dadosPrat){
        try {
            if (isset($dadosPrat)) {
                $this->entityManager->remove($dadosPrat);
                $this->entityManager->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function revisaReceitaLocal(
        $prato, $cdfilauxprat, $receO, $filialO, $servicoO, $filialD,
        $proporcao, $nrorg, $operador
    ) {
        try {
            $insert = "INSERT INTO RECEFILI
                        ( CDPRATO, CDFILAUXPRAT, CDFILIAL, CDPRODUTO,
                          QTPRODRECLOC, SGUNIDADE, VRPEFATCORLO,
                          VRPEFATCOCLO, HRTEMPPREPLO, QTUNIDESTLO,
                          QTBRUTALO, CDAPRERECE, QTBRUTALOUNR,
                          VRPEFATNUTLO, QTCALCNUTRLO, IDQTDEFIXLO,
                          IDPRODBASELO, DTULTATURECF, IDTPREQPRODFILI,
                          QTCOMPLPRODFILI, IDUTCOMPENTDIRF, QTLIMSEGFILI,
                          NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO,
                          IDATIVO, VRPERCDEGELO
                        )
                        (
                        SELECT CDPRATO, CDFILAUXPRAT, :P_CDFILIDEST AS CDFILIAL, CDPRODUTO,
                               (QTPRODPRAT * :P_PROPORCAO) AS QTPRODUTO, SGUNIDADE, VRPEFATCORPR AS VRPEFATCOR,
                               VRPEFATCOCPR AS VRPEFATCOC, HRTEMPPREPPR AS HRTEMPPREP, (QTUNIDESTPR * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTAPR * :P_PROPORCAO) AS QTBRUTA, CDAPRERECE, (QTBRUTAPRUNR * :P_PROPORCAO) AS QTBRUTAUNR,
                               VRPEFATNUTPR AS VRPEFATNUT, (QTCALCNUTRPR * :P_PROPORCAO) AS QTCALCNUTR, IDQTDEFIXPR,
                               IDPRODBASEPR AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODPADR AS IDTPREQPROD,
                               QTCOMPLPRODPADR AS QTCOMPL, IDUTCOMPENTDIRP AS IDUTCOMP, QTLIMSEGPADR AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECEPADR
                         WHERE (CDPRATO       = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (:P_IDTIPORECE = 'P')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECEFILI
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX
                                                      AND CDFILIAL     = :P_CDFILIDEST ))

                         UNION ALL

                        SELECT CDPRATO, CDFILAUXPRAT, :P_CDFILIDEST AS CDFILIAL,
                               CDPRODUTO, (QTPRODPRATTP * :P_PROPORCAO) AS QTPRODUTO,
                               SGUNIDADE, VRPEFATCORPRTP AS VRPEFATCOR,
                               VRPEFATCOCPRTP AS VRPEFATCOC, HRTEMPPREPPRTP AS HRTEMPPREP,
                               (QTUNIDESTPRTP * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTAPRTP * :P_PROPORCAO) AS QTBRUTA,
                               CDAPRERECE, (QTBRUTAPRUNRTP * :P_PROPORCAO) AS QTBRUTAUNR,
                               VRPEFATNUTPRTP AS VRPEFATNUT, (QTCALCNUTRPRTP * :P_PROPORCAO) AS QTCALCNUTR,
                               IDQTDEFIXPRTP,
                               IDPRODBASEPRTP AS IDPRODBASE, SYSDATE AS DTULTATU, 'I' AS IDTPREQPROD,
                               NULL AS QTCOMPL, 'N' AS IDUTCOMP, NULL AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECETPCARD
                         WHERE (CDPRATO = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (CDFILCENTPME  = :P_CDFILCENTPME)
                           AND (CDTIPOCARD    = :P_CDTIPOCARD)
                           AND (:P_IDTIPORECE = 'T')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECEFILI
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX ))

                         UNION ALL

                        SELECT CDPRATO, CDFILAUXPRAT, :P_CDFILIDEST   AS CDFILIAL,
                               CDPRODUTO, (QTPRODRECLOC * :P_PROPORCAO) AS QTPRODUTO,
                               SGUNIDADE, VRPEFATCORLO AS VRPEFATCOR,
                               VRPEFATCOCLO AS VRPEFATCOC, HRTEMPPREPLO AS HRTEMPPREP,
                               (QTUNIDESTLO * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTALO * :P_PROPORCAO) AS QTBRUTA,
                               CDAPRERECE, (QTBRUTALOUNR * :P_PROPORCAO) AS QTBRUTAUNR,
                               VRPEFATNUTLO AS VRPEFATNUT, (QTCALCNUTRLO * :P_PROPORCAO) AS QTCALCNUTR,
                               IDQTDEFIXLO,
                               IDPRODBASELO AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODFILI AS IDTPREQPROD,
                               QTCOMPLPRODFILI AS QTCOMPL, IDUTCOMPENTDIRF AS IDUTCOMP, QTLIMSEGFILI AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECEFILI
                         WHERE (CDPRATO       = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (CDFILIAL      = :P_CDFILIORIG)
                           AND (:P_IDTIPORECE = 'L')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECEFILI
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX
                                                      AND CDFILIAL = :P_CDFILIDEST ))

                        UNION ALL

                       SELECT CDPRATO, CDFILAUXPRAT, :P_CDFILIDEST   AS CDFILIAL,
                              CDPRODUTO, (QTPRODRECSER * :P_PROPORCAO) AS QTPRODUTO,
                              SGUNIDADE, VRPEFATCORSE AS VRPEFATCOR,
                              VRPEFATCOCSE AS VRPEFATCOC, HRTEMPPREPSE AS HRTEMPPREP,
                              (QTUNIDESTSE * :P_PROPORCAO) AS QTDERECE,
                              (QTBRUTASE * :P_PROPORCAO) AS QTBRUTA,
                              CDAPRERECE, (QTBRUTASEUNR * :P_PROPORCAO) AS QTBRUTAUNR,
                              VRPEFATNUTSE AS VRPEFATNUT, (QTCALCNUTRSE * :P_PROPORCAO) AS QTCALCNUTR,
                              IDQTDEFIXSE,
                              IDPRODBASESE AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODSERV AS IDTPREQPROD,
                              QTCOMPLPRODSERV AS QTCOMPL, IDUTCOMPENTDIRS AS IDUTCOMP, QTLIMSEGSERV AS QTLIM,
                              :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                              'S', VRPERCDEGELO
                         FROM RECESERV
                        WHERE (CDPRATO       = :P_CDPRATO)
                          AND (CDFILAUXPRAT  = :P_CDFILAUX)
                          AND (CDFILIAL      = :P_CDFILIORIG)
                          AND (CDSERVFILI    = :P_CDSERVORIG)
                          AND (:P_IDTIPORECE = 'S')
                          AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                    FROM RECEFILI
                                                   WHERE CDPRATO = :P_CDPRATO
                                                   AND CDFILAUXPRAT = :P_CDFILAUX
                                                   AND CDFILIAL = :P_CDFILIDEST ))
                      )";

            $params = array(
                "P_CDPRATO"=> $prato,
                "P_CDFILAUX"=> $cdfilauxprat,
                "P_IDTIPORECE"=> $receO,
                "P_CDFILIORIG"=> $filialO,
                "P_CDSERVORIG"=> $servicoO,
                "P_CDFILIDEST"=> $filialD,
                "P_PROPORCAO"=> $proporcao,
                "P_CDFILCENTPME"=> null,
                "P_CDTIPOCARD"=> null,
                "P_NRORG"=> $nrorg,
                "P_OPERADOR"=> $operador
            );
            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function revisaReceitaPadrao(
        $prato, $cdfilauxprat, $receO, $filialO, $servicoO,
        $proporcao, $nrorg, $operador
    ) {
        $proporcao = $proporcao ? $proporcao : 1;
        try {
            $insert = "INSERT INTO RECEPADR
                        ( CDPRATO, CDFILAUXPRAT, CDPRODUTO, QTPRODPRAT,
                          SGUNIDADE, VRPEFATCORPR, VRPEFATCOCPR, HRTEMPPREPPR,
                          QTUNIDESTPR, QTBRUTAPR, CDAPRERECE, QTBRUTAPRUNR,
                          VRPEFATNUTPR, QTCALCNUTRPR, IDQTDEFIXPR,

                          IDPRODBASEPR, DTULTATURECP, IDTPREQPRODPADR,
                          QTCOMPLPRODPADR, IDUTCOMPENTDIRP, QTLIMSEGPADR,
                          NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO,
                          IDATIVO, VRPERCDEGELO
                        )
                        (
                          SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO,
                                (QTPRODPRAT * :P_PROPORCAO) AS QTPRODUTO,
                                SGUNIDADE, VRPEFATCORPR AS VRPEFATCOR,
                                VRPEFATCOCPR AS VRPEFATCOC, HRTEMPPREPPR AS HRTEMPPREP,
                                (QTUNIDESTPR * :P_PROPORCAO) AS QTDERECE,
                                (QTBRUTAPR * :P_PROPORCAO) AS QTBRUTA, CDAPRERECE,
                                (QTBRUTAPRUNR * :P_PROPORCAO) AS QTBRUTAUNR,
                                VRPEFATNUTPR AS VRPEFATNUT, (QTCALCNUTRPR * :P_PROPORCAO) AS QTCALCNUTR,
                                IDQTDEFIXPR,
                                IDPRODBASEPR AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODPADR AS IDTPREQPROD,
                                QTCOMPLPRODPADR AS QTCOMPL, IDUTCOMPENTDIRP AS IDUTCOMP, QTLIMSEGPADR AS QTLIM,
                                :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                                'S', VRPERCDEGELO
                           FROM RECEPADR
                          WHERE (CDPRATO = :P_CDPRATO)
                            AND (CDFILAUXPRAT  = :P_CDFILAUX)
                            AND (:P_IDTIPORECE = 'P')
                            AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                      FROM RECEPADR
                                                     WHERE CDPRATO = :P_CDPRATO
                                                       AND CDFILAUXPRAT = :P_CDFILAUX ))

                          UNION ALL

                         SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO,
                               (QTPRODPRATTP * :P_PROPORCAO)  AS QTPRODUTO,
                               SGUNIDADE, VRPEFATCORPRTP AS VRPEFATCOR,
                               VRPEFATCOCPRTP AS VRPEFATCOC, HRTEMPPREPPRTP AS HRTEMPPREP,
                               (QTUNIDESTPRTP * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTAPRTP * :P_PROPORCAO) AS QTBRUTA, CDAPRERECE,
                               (QTBRUTAPRUNRTP * :P_PROPORCAO) AS QTBRUTAUNR,
                               VRPEFATNUTPRTP AS VRPEFATNUT, (QTCALCNUTRPRTP * :P_PROPORCAO) AS QTCALCNUTR,
                               IDQTDEFIXPRTP,
                               IDPRODBASEPRTP AS IDPRODBASE, SYSDATE AS DTULTATU, 'I' AS IDTPREQPROD,
                               NULL AS QTCOMPL, 'N' AS IDUTCOMP, NULL AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECETPCARD
                         WHERE (CDPRATO       = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (CDFILCENTPME  = :P_CDFILCENTPME)
                           AND (CDTIPOCARD    = :P_CDTIPOCARD)
                           AND (:P_IDTIPORECE = 'T')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECEPADR
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX ))

                         UNION ALL

                        SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO,
                               (QTPRODRECLOC * :P_PROPORCAO) AS QTPRODUTO,
                               SGUNIDADE, VRPEFATCORLO AS VRPEFATCOR,
                               VRPEFATCOCLO AS VRPEFATCOC, HRTEMPPREPLO AS HRTEMPPREP,
                               (QTUNIDESTLO * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTALO * :P_PROPORCAO) AS QTBRUTA, CDAPRERECE,
                               (QTBRUTALOUNR * :P_PROPORCAO) AS QTBRUTAUNR,
                               VRPEFATNUTLO AS VRPEFATNUT, (QTCALCNUTRLO * :P_PROPORCAO) AS QTCALCNUTR,
                               IDQTDEFIXLO,
                               IDPRODBASELO AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODFILI AS IDTPREQPROD,
                               QTCOMPLPRODFILI AS QTCOMPL, IDUTCOMPENTDIRF AS IDUTCOMP, QTLIMSEGFILI AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECEFILI
                         WHERE (CDPRATO       = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (CDFILIAL      = :P_CDFILIORIG)
                           AND (:P_IDTIPORECE = 'L')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECEPADR
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX ))

                         UNION ALL

                        SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO,
                               (QTPRODRECSER * :P_PROPORCAO) AS QTPRODUTO,
                               SGUNIDADE, VRPEFATCORSE AS VRPEFATCOR,
                               VRPEFATCOCSE AS VRPEFATCOC, HRTEMPPREPSE AS HRTEMPPREP,
                               (QTUNIDESTSE * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTASE * :P_PROPORCAO) AS QTBRUTA, CDAPRERECE,
                               (QTBRUTASEUNR * :P_PROPORCAO) AS QTBRUTAUNR,
                               VRPEFATNUTSE AS VRPEFATNUT, (QTCALCNUTRSE * :P_PROPORCAO) AS QTCALCNUTR,
                               IDQTDEFIXSE,
                               IDPRODBASESE AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODSERV AS IDTPREQPROD,
                               QTCOMPLPRODSERV AS QTCOMPL, IDUTCOMPENTDIRS AS IDUTCOMP, QTLIMSEGSERV AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECESERV
                         WHERE (CDPRATO       = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (CDFILIAL      = :P_CDFILIORIG)
                           AND (CDSERVFILI    = :P_CDSERVORIG)
                           AND (:P_IDTIPORECE = 'S')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECEPADR
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX ))
                                                   )";

            $params = array(
                "P_PROPORCAO"=> $proporcao,
                "P_CDPRATO"=> $prato,
                "P_CDFILAUX"=> $cdfilauxprat,
                "P_IDTIPORECE"=> $receO,
                "P_CDFILIORIG"=> $filialO,
                "P_CDSERVORIG"=> $servicoO,
                "P_CDFILCENTPME"=> 'X',
                "P_CDTIPOCARD"=> 'X',
                "P_NRORG"=> $nrorg,
                "P_OPERADOR"=> $operador
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function revisaReceitaServico(
        $prato, $cdfilauxprat, $receO, $filialO, $servicoO, $proporcao,
        $filialD, $servicoD, $nrorg, $operador
    ) {
        try {
            $insert = "INSERT INTO RECESERV
                        ( CDPRATO, CDFILAUXPRAT, CDFILIAL, CDSERVFILI,
                          CDPRODUTO, QTPRODRECSER, SGUNIDADE, VRPEFATCORSE,
                          VRPEFATCOCSE, HRTEMPPREPSE, QTUNIDESTSE, QTBRUTASE,
                          CDAPRERECE, QTBRUTASEUNR, VRPEFATNUTSE, QTCALCNUTRSE,
                          IDQTDEFIXSE,
                          IDPRODBASESE, DTULTATURECS, IDTPREQPRODSERV,
                          QTCOMPLPRODSERV, IDUTCOMPENTDIRS, QTLIMSEGSERV, NRORG,
                          DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, IDATIVO, VRPERCDEGELO
                        )
                        (
                          SELECT CDPRATO, CDFILAUXPRAT, :P_CDFILIDEST AS CDFILIAL,
                                :P_CDSERVDEST AS CDSERVFILI, CDPRODUTO,
                                (QTPRODPRAT * :P_PROPORCAO) AS QTPRODUTO,
                                SGUNIDADE, VRPEFATCORPR AS VRPEFATCOR,
                                VRPEFATCOCPR AS VRPEFATCOC, HRTEMPPREPPR AS HRTEMPPREP,
                                (QTUNIDESTPR * :P_PROPORCAO) AS QTDERECE,
                                (QTBRUTAPR * :P_PROPORCAO) AS QTBRUTA,
                                CDAPRERECE, (QTBRUTAPRUNR * :P_PROPORCAO) AS QTBRUTAUNR,
                                VRPEFATNUTPR AS VRPEFATNUT, (QTCALCNUTRPR * :P_PROPORCAO) AS QTCALCNUTR,
                                IDQTDEFIXPR,
                                IDPRODBASEPR AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODPADR AS IDTPREQPROD,
                                QTCOMPLPRODPADR AS QTCOMPL, IDUTCOMPENTDIRP AS IDUTCOMP, QTLIMSEGPADR AS QTLIM,
                                :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                                'S', VRPERCDEGELO
                           FROM RECEPADR
                          WHERE (CDPRATO       = :P_CDPRATO)
                            AND (CDFILAUXPRAT  = :P_CDFILAUX)
                            AND (:P_IDTIPORECE = 'P')
                            AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                      FROM RECESERV
                                                     WHERE CDPRATO = :P_CDPRATO
                                                       AND CDFILAUXPRAT = :P_CDFILAUX
                                                       AND CDFILIAL = :P_CDFILIDEST
                                                       AND CDSERVFILI   = :P_CDSERVDEST ))

                         UNION ALL

                        SELECT CDPRATO, CDFILAUXPRAT, :P_CDFILIDEST AS CDFILIAL,
                               :P_CDSERVDEST AS CDSERVFILI, CDPRODUTO,
                               (QTPRODPRATTP * :P_PROPORCAO) AS QTPRODUTO,
                               SGUNIDADE, VRPEFATCORPRTP AS VRPEFATCOR,
                               VRPEFATCOCPRTP AS VRPEFATCOC, HRTEMPPREPPRTP AS HRTEMPPREP,
                               (QTUNIDESTPRTP * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTAPRTP * :P_PROPORCAO) AS QTBRUTA,
                               CDAPRERECE, (QTBRUTAPRUNRTP * :P_PROPORCAO) AS QTBRUTAUNR,
                               VRPEFATNUTPRTP AS VRPEFATNUT,
                               (QTCALCNUTRPRTP * :P_PROPORCAO) AS QTCALCNUTR,
                               IDQTDEFIXPRTP,
                               IDPRODBASEPRTP AS IDPRODBASE, SYSDATE AS DTULTATU, 'I' AS IDTPREQPROD,
                               NULL AS QTCOMPL, 'N' AS IDUTCOMP, NULL AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECETPCARD
                         WHERE (CDPRATO       = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (CDFILCENTPME  = :P_CDFILCENTPME)
                           AND (CDTIPOCARD    = :P_CDTIPOCARD)
                           AND (:P_IDTIPORECE = 'T')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECESERV
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX ))
                         UNION ALL

                        SELECT CDPRATO, CDFILAUXPRAT, :P_CDFILIDEST AS CDFILIAL,
                               :P_CDSERVDEST AS CDSERVFILI, CDPRODUTO,
                               (QTPRODRECLOC * :P_PROPORCAO) AS QTPRODUTO,
                               SGUNIDADE, VRPEFATCORLO AS VRPEFATCOR,
                               VRPEFATCOCLO AS VRPEFATCOC, HRTEMPPREPLO AS HRTEMPPREP,
                               (QTUNIDESTLO * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTALO * :P_PROPORCAO) AS QTBRUTA, CDAPRERECE,
                               (QTBRUTALOUNR * :P_PROPORCAO)  AS QTBRUTAUNR,
                               VRPEFATNUTLO AS  VRPEFATNUT, (QTCALCNUTRLO * :P_PROPORCAO) AS QTCALCNUTR,
                               IDQTDEFIXLO,
                               IDPRODBASELO AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODFILI AS IDTPREQPROD,
                               QTCOMPLPRODFILI AS QTCOMPL, IDUTCOMPENTDIRF AS IDUTCOMP, QTLIMSEGFILI AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECEFILI
                         WHERE (CDPRATO       = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (CDFILIAL      = :P_CDFILIORIG)
                           AND (:P_IDTIPORECE = 'L')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECESERV
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX
                                                      AND CDFILIAL = :P_CDFILIDEST
                                                      AND CDSERVFILI = :P_CDSERVDEST ))

                         UNION ALL

                        SELECT CDPRATO, CDFILAUXPRAT, :P_CDFILIDEST AS CDFILIAL,
                               :P_CDSERVDEST AS CDSERVFILI, CDPRODUTO,
                               (QTPRODRECSER * :P_PROPORCAO) AS QTPRODUTO,
                               SGUNIDADE, VRPEFATCORSE AS VRPEFATCOR,
                               VRPEFATCOCSE AS VRPEFATCOC, HRTEMPPREPSE AS HRTEMPPREP,
                               (QTUNIDESTSE * :P_PROPORCAO) AS QTDERECE,
                               (QTBRUTASE * :P_PROPORCAO) AS QTBRUTA, CDAPRERECE,
                               (QTBRUTASEUNR * :P_PROPORCAO) AS QTBRUTAUNR,
                               VRPEFATNUTSE AS VRPEFATNUT, (QTCALCNUTRSE * :P_PROPORCAO) AS QTCALCNUTR,
                               IDQTDEFIXSE,
                               IDPRODBASESE AS IDPRODBASE, SYSDATE AS DTULTATU, IDTPREQPRODSERV AS IDTPREQPROD,
                               QTCOMPLPRODSERV AS QTCOMPL, IDUTCOMPENTDIRS AS IDUTCOMP, QTLIMSEGSERV AS QTLIM,
                               :P_NRORG AS NRORG, SYSDATE AS DTINCLUSAO, :P_NRORG AS NRORGINCLUSAO, :P_OPERADOR AS CDOPERINCLUSAO,
                               'S', VRPERCDEGELO
                          FROM RECESERV
                         WHERE (CDPRATO       = :P_CDPRATO)
                           AND (CDFILAUXPRAT  = :P_CDFILAUX)
                           AND (CDFILIAL      = :P_CDFILIORIG)
                           AND (CDSERVFILI    = :P_CDSERVORIG)
                           AND (:P_IDTIPORECE = 'S')
                           AND (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                                     FROM RECESERV
                                                    WHERE CDPRATO = :P_CDPRATO
                                                      AND CDFILAUXPRAT = :P_CDFILAUX
                                                      AND CDFILIAL = :P_CDFILIDEST
                                                      AND CDSERVFILI = :P_CDSERVDEST )))";

            $params = array(
                "P_PROPORCAO"=> $proporcao,
                "P_CDPRATO"=> $prato,
                "P_CDFILAUX"=> $cdfilauxprat,
                "P_IDTIPORECE"=> $receO,
                "P_CDFILIORIG"=> $filialO,
                "P_CDSERVORIG"=> $servicoO,
                "P_CDFILIDEST"=> $filialD,
                "P_CDSERVDEST"=> $servicoD,
                "P_CDFILCENTPME"=> null,
                "P_CDTIPOCARD"=> null,
                "P_NRORG"=> $nrorg,
                "P_OPERADOR"=> $operador
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }


    public function retornaProdutoProducao($prato, $filauxprat, $produto, $tProduto, $nrorg){
        $sql = "SELECT P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.CDNVPRODUTO, P.IDPRODPROD
                FROM PRODUTO P
                     LEFT JOIN
                     (SELECT DISTINCT CDPRODPRODUCAO
                        FROM PRATPADR
                       WHERE CDPRATO <> :CDPRATO OR CDFILAUXPRAT <> :CDFILAUXPRAT
                       UNION
                      SELECT DISTINCT CDPRODPRODUCAO
                        FROM PRATFILI
                       WHERE CDPRATO <> :CDPRATO OR CDFILAUXPRAT <> :CDFILAUXPRAT
                       UNION
                      SELECT DISTINCT CDPRODPRODUCAO
                        FROM PRATSERV
                       WHERE CDPRATO <> :CDPRATO OR CDFILAUXPRAT <> :CDFILAUXPRAT
                     ) PP ON PP.CDPRODPRODUCAO = P.CDPRODUTO
                WHERE ((P.CDARVPROD = :CDPRODUTO) OR (:TCDPRODUTO = 'T')) AND
                      (P.IDPRODPROD = 'S') AND
                      (PP.CDPRODPRODUCAO IS NULL)
                ORDER BY P.CDARVPROD";

        $params = array(
            "CDPRATO" => $prato,
            "CDFILAUXPRAT" => $filauxprat,
            "CDPRODUTO" => $produto,
            "TCDPRODUTO" => $tProduto,
            "NRORG" => $nrorg
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificarProdutoProducao($prato, $filauxprat, $produto, $receita, $nrorg){
        $sql = "SELECT A.TIPO, A.CDFILIAL, NVL(F.NMFILIAL,'') AS NMFILIAL, NVL(A.CDSERVFILI,'') AS CDSERVFILI, NVL(S.NMSERVICO,'') AS NMSERVICO, A.CDPRATO, P.NMPRATO, A.CDPRODPRODUCAO
                FROM
                (SELECT 0 AS ID, 'P' AS TIPO, '' AS CDFILIAL, '' AS CDSERVFILI, CDPRATO, CDFILAUXPRAT, CDPRODPRODUCAO
                FROM PRATPADR WHERE CDPRODPRODUCAO = :CDPRODPRODUCAO
                 AND ((:RECEITA <> 'P') OR ((:RECEITA = 'P') AND (CDPRATO||CDFILAUXPRAT <> :CDPRATO||:CDFILAUXPRAT)))
                UNION ALL
                SELECT 1 AS ID, 'F' AS TIPO, CDFILIAL, '' AS CDSERVFILI, CDPRATO, CDFILAUXPRAT, CDPRODPRODUCAO
                FROM PRATFILI WHERE CDPRODPRODUCAO = :CDPRODPRODUCAO
                 AND ((:RECEITA <> 'F') OR ((:RECEITA = 'F') AND (CDPRATO||CDFILAUXPRAT <> :CDPRATO||:CDFILAUXPRAT)))
                UNION ALL
                SELECT 2 AS ID, 'S' AS TIPO, CDFILIAL, CDSERVFILI, CDPRATO, CDFILAUXPRAT, CDPRODPRODUCAO
                FROM PRATSERV WHERE CDPRODPRODUCAO = :CDPRODPRODUCAO
                 AND ((:RECEITA <> 'S') OR ((:RECEITA = 'S') AND (CDPRATO||CDFILAUXPRAT <> :CDPRATO||:CDFILAUXPRAT)))) A
                LEFT JOIN FILIAL F ON A.CDFILIAL = F.CDFILIAL
                LEFT JOIN SERVICO S ON S.CDFILIAL = A.CDFILIAL AND S.CDSERVFILI = A.CDSERVFILI
                LEFT JOIN PRATO P ON P.CDPRATO = A.CDPRATO AND P.CDFILAUXPRAT = A.CDFILAUXPRAT
                ORDER BY A.ID, A.CDFILIAL, A.CDSERVFILI";

        $params = array(
            "CDPRATO"        => $prato,
            "CDFILAUXPRAT"   => $filauxprat,
            "CDPRODPRODUCAO" => $produto,
            "RECEITA"        => $receita,
            "NRORG"          => $nrorg
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaPratoGramagem($cdprato, $cdFilAuxPrat, $filial, $servico){
        $dado = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Pratogramag')
            ->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab(),
                'cdprato' => $cdprato,
                'cdfilauxprat' => $cdFilAuxPrat,
                'cdfilial' => $filial,
                'cdservfili' => $servico));
                
        return $dado;
    }

    public function excluiPratoGramagem($registro){
        try {
            if (isset($registro)) {
                $this->entityManager->remove($registro);
                $this->entityManager->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function retornaProdutoPorPratos($row){

        $sql = "SELECT DECODE(FILIAL,'P',:TEXTO_PADRAO,FILIAL) AS RECEITA,
    FILIAL,
    NMTABLE,
    DECODE(R.CDFILAUXPRAT,'P',:TEXTO_PADRAO,R.CDFILAUXPRAT) AS ARVORE, R.CDFILAUXPRAT, R.CDPRATO, PR.NMPRATO, CDSERVFILI, NMSERVICO, CDARVPROD, NMPRODUTO,
                       QTPRODPRAT, R.SGUNIDADE, VRPEFATCOCPR, VRPEFATCOCPRAP, VRPEFATCORPR, VRPEFATCORPRAP, HRTEMPPREPPR,
                       QTBRUTAPRUNR, QTBRUTAPR, IDPRODBASEPR, VRPEFATNUTPR, VRPERCDEGELO, QTCALCNUTRPR, QTUNIDESTPR, CDAPRERECE,
                       DTULTATURECP, SGUNIDPROD, CDTIPOCARD, CDPRODUTO, CDFILCENTPME,
                       RPAD('N', 1, ' ') IDMARCADO
                  FROM (SELECT 'P' AS FILIAL, 'RECEPADR' NMTABLE, RP.CDFILAUXPRAT, RP.CDPRATO, '' CDSERVFILI, '' NMSERVICO,
                               PD.CDARVPROD, PD.NMPRODUTO, RP.QTPRODPRAT, RP.SGUNIDADE, RP.VRPEFATCOCPR,
                               (RP.VRPEFATCOCPR + 100) AS VRPEFATCOCPRAP, RP.VRPEFATCORPR, (100 - RP.VRPEFATCORPR) AS VRPEFATCORPRAP, RP.VRPERCDEGELO,
                               RP.HRTEMPPREPPR, RP.QTBRUTAPRUNR, RP.QTBRUTAPR, NVL(RP.IDPRODBASEPR, 'N') IDPRODBASEPR,
                               RP.VRPEFATNUTPR, RP.QTCALCNUTRPR, RP.QTUNIDESTPR, RP.CDAPRERECE, RP.DTULTATURECP,
                               PD.SGUNIDADE AS SGUNIDPROD, '' CDTIPOCARD, RP.CDPRODUTO, '' AS CDFILCENTPME
                          FROM PRODUTO PD, RECEPADR RP
                         WHERE (RP.CDFILAUXPRAT = :P_CDFILAUXPRAT)
                           AND RP.CDPRATO BETWEEN :P_CDPRATO1 AND :P_CDPRATO2
                           AND (SUBSTR(PD.CDARVPROD, 1, LENGTH(:P_CDPRODUTO)) = :P_CDPRODUTO OR :P_CDPRODUTO = 'TODOS')
                           AND RP.CDPRODUTO = PD.CDPRODUTO
                           AND :P_RECEPADR   = 'S'
                         UNION ALL
                         SELECT DISTINCT 'C' AS FILIAL, 'RECETPCARD' NMTABLE, RC.CDFILAUXPRAT, RC.CDPRATO, '' CDSERVFILI, '' NMSERVICO,
                                PD.CDARVPROD, PD.NMPRODUTO, RC.QTPRODPRATTP, RC.SGUNIDADE, RC.VRPEFATCOCPRTP,
                                (RC.VRPEFATCOCPRTP + 100) AS VRPEFATCOCPRAP, RC.VRPEFATCORPRTP, (100 - RC.VRPEFATCORPRTP) AS VRPEFATCORPRAP, RC.VRPERCDEGELO, 
                                RC.HRTEMPPREPPRTP, RC.QTBRUTAPRUNRTP, RC.QTBRUTAPRTP, NVL(RC.IDPRODBASEPRTP, 'N') IDPRODBASEPR,
                                RC.VRPEFATNUTPRTP, RC.QTCALCNUTRPRTP, RC.QTUNIDESTPRTP, RC.CDAPRERECE, RC.DTULTATURECPTP,
                                PD.SGUNIDADE AS SGUNIDPROD, RC.CDTIPOCARD, RC.CDPRODUTO, RC.CDFILCENTPME
                           FROM PRODUTO PD, RECETPCARD RC, FILITIPOCARD FT,
                                (SELECT SUBSTR(CDLISTA, 1, 4) CDFILIAL
                                   FROM LISTA
                                  WHERE NRLISTA = :P_NRLISTASERV
                                  GROUP BY SUBSTR(CDLISTA, 1, 4)) F
                          WHERE ((RC.CDFILAUXPRAT = 'P') OR (RC.CDFILAUXPRAT = F.CDFILIAL))
                           AND RC.CDPRATO BETWEEN :P_CDPRATO1 AND :P_CDPRATO2
                           AND (SUBSTR(PD.CDARVPROD, 1, LENGTH(:P_CDPRODUTO)) = :P_CDPRODUTO OR :P_CDPRODUTO = 'TODOS')
                           AND (RC.CDPRODUTO = PD.CDPRODUTO)
                           AND (RC.CDFILCENTPME = :P_CDFILCENTPME)
                           AND ((RC.CDTIPOCARD = :P_CDTIPOCARD) OR (:P_CDTIPOCARD = 'T'))
                           AND (RC.CDFILCENTPME = FT.CDFILCENTPME)
                           AND (RC.CDTIPOCARD = FT.CDTIPOCARD)
                           AND (:P_RECETPCARD   = 'S')
                           AND ((FT.CDFILIAL = :P_CDFILIAL) OR (FT.CDFILCENTPME = :P_CDFILIAL))
                         UNION ALL
                         SELECT RF.CDFILIAL AS FILIAL, 'RECEFILI' NMTABLE, RF.CDFILAUXPRAT, RF.CDPRATO, '' CDSERVFILI,
                                '' NMSERVICO, PD.CDARVPROD, PD.NMPRODUTO, RF.QTPRODRECLOC QTPRODPRAT, RF.SGUNIDADE,
                                RF.VRPEFATCOCLO VRPEFATCOCPR, (RF.VRPEFATCOCLO + 100) AS VRPEFATCOCPRAP, RF.VRPEFATCORLO VRPEFATCORPR, 
                                (100 - RF.VRPEFATCORLO) AS VRPEFATCORPRAP, RF.VRPERCDEGELO, 
                                RF.HRTEMPPREPLO AS HRTEMPPREPPR,
                                RF.QTBRUTALOUNR AS QTBRUTAPRUNR, RF.QTBRUTALO AS QTBRUTAPR, NVL(RF.IDPRODBASELO,'N') AS IDPRODBASEPR,
                                RF.VRPEFATNUTLO AS VRPEFATNUTPR, RF.QTCALCNUTRLO AS QTCALCNUTRPR, RF.QTUNIDESTLO  AS QTUNIDESTPR,
                                RF.CDAPRERECE, RF.DTULTATURECF AS DTULTATURECP, PD.SGUNIDADE AS SGUNIDPROD, '' CDTIPOCARD, RF.CDPRODUTO,
                                '' AS CDFILCENTPME
                           FROM PRODUTO PD, RECEFILI RF,
                                (SELECT SUBSTR(CDLISTA, 1, 4) CDFILIAL
                                   FROM LISTA
                                  WHERE NRLISTA = :P_NRLISTASERV
                                 GROUP BY SUBSTR(CDLISTA, 1, 4)) F
                          WHERE (SUBSTR(PD.CDARVPROD, 1, LENGTH(:P_CDPRODUTO)) = :P_CDPRODUTO OR :P_CDPRODUTO = 'TODOS')
                            AND RF.CDPRATO BETWEEN :P_CDPRATO1 AND :P_CDPRATO2
                            AND ((RF.CDFILAUXPRAT = 'P') OR (RF.CDFILAUXPRAT = F.CDFILIAL))
                            AND ((RF.CDFILIAL     = F.CDFILIAL))
                            AND RF.CDPRODUTO      = PD.CDPRODUTO
                            AND :P_RECEFILI       = 'S'
                          UNION ALL
                         SELECT RS.CDFILIAL AS FILIAL, 'RECESERV' NMTABLE, RS.CDFILAUXPRAT, RS.CDPRATO, RS.CDSERVFILI,
                                SE.NMSERVICO, PD.CDARVPROD, PD.NMPRODUTO, RS.QTPRODRECSER QTPRODPRAT, RS.SGUNIDADE,
                                RS.VRPEFATCOCSE VRPEFATCOCPR, (RS.VRPEFATCOCSE + 100) AS VRPEFATCOCPRAP, RS.VRPEFATCORSE VRPEFATCORPR, 
                                (100 - RS.VRPEFATCORSE) AS VRPEFATCORPRAP, RS.VRPERCDEGELO, 
                                RS.HRTEMPPREPSE AS HRTEMPPREPPR,
                                RS.QTBRUTASEUNR AS QTBRUTAPRUNR, RS.QTBRUTASE AS QTBRUTAPR, NVL(RS.IDPRODBASESE, 'N') AS IDPRODBASEPR,
                                RS.VRPEFATNUTSE AS VRPEFATNUTPR, RS.QTCALCNUTRSE AS QTCALCNUTRPR, RS.QTUNIDESTSE AS QTUNIDESTPR,
                                RS.CDAPRERECE, RS.DTULTATURECS AS DTULTATURECP, PD.SGUNIDADE AS SGUNIDPROD, '' CDTIPOCARD, RS.CDPRODUTO,
                                '' AS CDFILCENTPME
                           FROM PRODUTO PD, SERVICO SE, RECESERV RS,
                                (SELECT SUBSTR(CDLISTA, 1, 4) CDFILIAL, SUBSTR(CDLISTA, 5, 5) CDSERVFILI
                                   FROM LISTA
                                  WHERE NRLISTA = :P_NRLISTASERV) F
                          WHERE (SUBSTR(PD.CDARVPROD, 1, LENGTH(:P_CDPRODUTO)) = :P_CDPRODUTO OR :P_CDPRODUTO = 'TODOS')
                            AND (RS.CDPRATO BETWEEN :P_CDPRATO1 AND :P_CDPRATO2)
                            AND ((RS.CDFILAUXPRAT = 'P') OR (RS.CDFILAUXPRAT = F.CDFILIAL))
                            AND (RS.CDFILIAL      = F.CDFILIAL)
                            AND (RS.CDSERVFILI    = F.CDSERVFILI)
                            AND (RS.CDFILIAL      = SE.CDFILIAL)
                            AND (RS.CDSERVFILI    = SE.CDSERVFILI)
                            AND (:P_RECESERV      = 'S')
                            AND RS.CDPRODUTO      = PD.CDPRODUTO) R, PRATO PR
                 WHERE PR.CDFILAUXPRAT  = R.CDFILAUXPRAT
                   AND PR.CDPRATO       = R.CDPRATO
                 ORDER BY R.CDPRATO, R.CDFILAUXPRAT, FILIAL, CDTIPOCARD, CDSERVFILI, CDARVPROD";

        $params = array(
            "P_CDFILAUXPRAT" => $row['P_CDFILAUXPRAT'],
            "P_CDPRATO1"     => $row['P_CDPRATO1'],
            "P_CDPRATO2"     => $row['P_CDPRATO2'],
            "P_CDPRODUTO"    => $row['P_CDPRODUTO'],
            "P_NRLISTASERV"  => $row['P_NRLISTASERV'],
            "P_CDFILCENTPME" => $row['P_CDFILCENTPME'],
            "P_CDTIPOCARD"   => $row['P_CDTIPOCARD'],
            "P_CDFILIAL"     => $row['P_CDFILIAL'],
            "P_RECEFILI"     => $row['P_RECEFILI'],
            "P_RECETPCARD"   => $row['P_RECETPCARD'],
            "P_RECEPADR"     => $row['P_RECEPADR'],
            "P_RECESERV"     => $row['P_RECESERV'],
            "TEXTO_PADRAO"   => $row['TEXTO_PADRAO']
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaEstoque($row) {
        $query = "SELECT CDPRODUTO, IDCNTRESTOQ FROM PRODFILI WHERE CDFILIAL = :CDFILIAL AND CDPRODUTO = :CDPRODUTO";
        $params = array(
            "CDFILIAL" => isset($row['CDFILIAL']) ? $row['CDFILIAL'] : $this->environment->getCdFilial(),
            "CDPRODUTO" => $row['CDPRODUTO']
        );
        $result = $this->entityManager->getConnection()->fetchAll($query, $params);
        if(isset($result[0]) && isset($result[0]['IDCNTRESTOQ']) && $result[0]['IDCNTRESTOQ'] == "N")
            return false;
        return true;
    }
    
    public function validaImportacaoReceita(&$row){
        if(isset($row['NOVOSPRODUTOS'])){
            $strProdutos = '';
            foreach($row['NOVOSPRODUTOS'] as $produto){
                $strProdutos .= '_'.$produto['CDPRODUTO'];
            }
            $query = "SELECT IDCNTRESTOQ, IDPRODATIV FROM PRODFILI WHERE VALOR_NA_LISTA(CDPRODUTO, :CDPRODUTO) <> 0 AND CDFILIAL = :CDFILIAL AND (IDCNTRESTOQ = 'N' OR IDPRODATIV = 'I')";
            $params = array(
                "CDFILIAL" => $row['FILIAL_D'],
                "CDPRODUTO" => $strProdutos
            );
            $result = $this->entityManager->getConnection()->fetchAll($query, $params);
            if(empty($result)){
                return true;
            }else{
                foreach($result as $produto){
                    if($produto['IDPRODATIV'] == 'I'){
                        $row['prodInativo'] = true;
                    }
                    if($produto['IDCNTRESTOQ'] == 'N'){
                        $row['estoque'] = true;
                    }
                }
                return false;
            }
            return true;
        }else{
            return false;
        }
    }
    
    public function getCdPictPrat(){
        $query = "SELECT CDPICTPRAT FROM PARAMGERAL";
        $connection =  $this->entityManager->getConnection();
        $stmt = $connection->prepare($query);
        $stmt->execute();
        return $stmt->fetch()['CDPICTPRAT'];
    }

    public function getFormatCodePlate($code, $mask){
        $regex = '';
        $toReplace = '';
        preg_match_all("/[^\.]+/", $mask, $results);
        $results = $results[0];
        for($i = 0; $i < sizeof($results); $i++){
            $sizeMatch = strlen($results[$i]);
            $regex .= $i === 0 ? "(.{{$sizeMatch}})" : "(.{0,$sizeMatch})";
            $toReplace .= $i !== 0 ? '.' : $toReplace;
            $toReplace .= "$".($i+1);
        }
        $formatCode = preg_replace("/".$regex."/", $toReplace, $code);
        $cleanCode = preg_replace("/(\.+)$/","",$formatCode);
        return $cleanCode;
    }
    
    public function inserePratoPadrao($info){
        try {
            $insert = "INSERT INTO PRATO (
                         CDFILAUXPRAT, CDPRATO, NMPRATO, 
                         CDNVPRATO, SGUNIDADE, NRORG)
                       VALUES (
                         :CDFILAUXPRAT, :CDPRATO, :NMPRATO, 
                         :CDNVPRATO, :SGUNIDADE, :NRORG)";

            $params = array(
                "CDFILAUXPRAT"=> $info['CDFILAUXPRAT'],
                "CDPRATO"=> $info['CDPRATO'],
                "NMPRATO"=> $info['NMPRATO'],
                "CDNVPRATO"=> $info['CDNVPRATO'],
                "SGUNIDADE"=> $info['SGUNIDADE'],
                "NRORG"=> $info['NRORG']
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function excluiReceitaCompl($nmTabelaReceita, $nmTabelaDadosRec, $cdprato, $cdfilauxprat){
        $sqlReceita = "DELETE FROM $nmTabelaReceita WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO";
        $sqlDadosRec = "DELETE FROM $nmTabelaDadosRec WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO";

        $params = array(
            "CDPRATO"=>$cdprato,
            "CDFILAUXPRAT" => $cdfilauxprat
        );
        try {
            $query = $this->entityManager->getConnection()->prepare($sqlReceita);
            $query->execute($params);
            
            $query = $this->entityManager->getConnection()->prepare($sqlDadosRec);
            $query->execute($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insereReceitaPadrao($info){
        try {
            /*
            VRPEFATCOCPR = 0
            QTBRUTAPRUNR E QTUNIDESTPR E QTBRUTAPR = QTPRODUTO
            QTCALCNUTRPR E QTPRODPRAT = QTPRODUTO - QTPERDA
            IDTPREQPRODPADR = P
            IDUTCOMPENTDIRP = N
            VRPEFATNUTPR = 100
            
            */
            if ($info['QTPERDA'] == 0) {
                $qtproduto = $info['QTPRODUTO'];
            } else {
                $qtproduto = $info['QTPRODUTO'] - ($info['QTPRODUTO'] * ($info['QTPERDA'] / 100));
            }
            
            $insert = "INSERT INTO RECEPADR (
                            CDFILAUXPRAT, CDPRATO, CDPRODUTO, QTPRODPRAT, VRPEFATCOCPR, 
                            VRPEFATCORPR, SGUNIDADE, QTUNIDESTPR, QTBRUTAPR, 
                            VRPEFATNUTPR, QTCALCNUTRPR, QTBRUTAPRUNR, IDTPREQPRODPADR, 
                            IDUTCOMPENTDIRP, NRORG, DTINCLUSAO, DTULTATU, CDOPERINCLUSAO, 
                            CDOPERULTATU, IDATIVO, DTULTATURECP) 
                        VALUES (
                            :CDFILAUXPRAT, :CDPRATO, :CDPRODUTO, :QTPRODPRAT, 0, 
                            :VRPEFATCORPR, :SGUNIDADE, :QTUNIDESTPR, :QTUNIDESTPR,
                            100, :QTPRODPRAT, :QTUNIDESTPR, 'P', 
                            'N', :NRORG, SYSDATE, SYSDATE, :CDOPERADOR, 
                            :CDOPERADOR, 'S', SYSDATE) ";

            $params = array(
                "CDFILAUXPRAT"=> $info['CDFILAUXPRAT'],
                "CDPRATO"=> $info['CDPRATO'],
                "CDPRODUTO"=> $info['CDPRODUTO'],
                "QTPRODPRAT"=> $qtproduto,
                "VRPEFATCORPR"=> $info['QTPERDA'],
                "QTUNIDESTPR"=> $info['QTPRODUTO'],
                "NRORG"=> $info['NRORG'],
                "SGUNIDADE"=> $info['SGUNIPROD'],
                "CDOPERADOR"=> $info['CDOPERADOR']
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaPrato($filAux, $prato, $nmPrato, $sgunidade) {
        try {
            $update = "UPDATE PRATO
                          SET NMPRATO = :NMPRATO,
                              SGUNIDADE = :SGUNIDADE
                        WHERE CDFILAUXPRAT = :CDFILAUXPRAT
                          AND CDPRATO = :CDPRATO";

            $params = array(
                "CDFILAUXPRAT"=> $filAux,
                "CDPRATO"=> $prato,
                "NMPRATO"=> $nmPrato,
                "SGUNIDADE"=> $sgunidade
            );

            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function hasCompleteLevel($cdprato, $cdpictprat){
        $cdpictprat = str_replace('.', '', $cdpictprat);
        return strlen($cdprato) == strlen($cdpictprat);
    }

    function isEmptyOrBlank($value){
        return !isset($value) || $this->isBlank($value);
    }
    
    function isBlank($value){
        $newStr = str_replace(" ", "", $value);
        return strlen($newStr) === 0;
    }
    
    function valueToFloat($value){
        if(is_string($value) && !$this->isEmptyOrBlank($value)){
            $dotPos   = strpos($value, ".");
            $commaPos = strpos($value, ",");
            if($commaPos == false && $dotPos == false){
                return floatval($value);
            } 
            $dotPos   = $dotPos   !== false ? $dotPos   : -1;
            $commaPos = $commaPos !== false ? $commaPos : -1;
            if($commaPos > $dotPos){
                $firstReplace = str_replace(".", "", $value);
                $secondReplace = str_replace(",", ".", $firstReplace);
                return floatval($secondReplace);
            } 
            if($commaPos < $dotPos){
                $firstReplace = str_replace(",", "", $value);
                return floatval($firstReplace);
            }
        } else {
            return $value;
        }
    }

    function isNumeric($value){
        $pattern = "/[0-9,.]+/i";
        $replacement = "";
        $newValue = preg_replace($pattern, $replacement, $value);
        return empty($newValue);
    }
    
    public function formatValue($value, $columnName, $cdpictprat = NULL){
        $erro = array();   
        switch($columnName){
            case 'CDPRATO':
                $value = str_replace('.', '', $value); 
                $value = str_replace(',', '', $value);
                $value = str_replace(' ', '', $value);
                
                if (strlen($value) == 0) $erro[] = $this->CDPRATO_N_INFO; 
                else if (strlen($value) > 10) {
                    $erro[] = $this->TAM_CODIGO;
                } else {
                    $verifMasc = $this->verificaMascPrato($value);
                    if ($verifMasc === false){
                        $erro[] = $this->TAM_CODIGO_PRATO;
                    } else if ($verifMasc === $this->MASC_N_DEFINIDA) {
                        $erro[] = $verifMasc;
                    }
                }
                break;
            case 'CDPRODUTO': 
                $produto = $value['CDPRODUTO'];
                $unidade = $value['SGUNIPROD'];
                $qtd = $value['QTPRODUTO'];
                if($this->isNumeric($qtd)){
                    $qtd = $this->valueToFloat($qtd);
                }
                //verifica se o prato é de nível completo.
                $pratCompleto = $cdpictprat ?? $this->getCdPictPrat();
                
                if($this->hasCompleteLevel($value['CDPRATO'], $pratCompleto) AND strlen($produto) == 0){
                    $erro[] = $this->PROD_N_INFO;
                }
                if (strlen($produto) > 0) {
                    //valida produto
                    $produto = str_replace('.', '', $produto); 
                    $produto = str_replace(',', '', $produto);
                    $produto = str_replace(' ', '', $produto); 
                    
                    if (strlen($produto) > 10) {
                        $erro[] = $this->TAM_CODIGO;
                       
                    } else {
                        $prodExiste = $this->verificaProduto($produto);
                        if (!$prodExiste OR count($prodExiste) == 0) $erro[] = $this->PRODUTO_N_EXISTE;
                        else { 
                            if($produto != null){
                                foreach($value['CDPRODUTOANT'] as $key){
                                    if($produto == $key['CDPRODUTO'] && $value['CDPRATO'] == $key['CDPRATO']) 
                                        $erro[] = $this->PROD_JA_INFO;
                                }
                            }
                            //valida unidade se produto existir
                            if (strlen($unidade) > 0) {
                                $unidade = strtoupper($unidade);  
                                if (strlen($unidade) > 10) {
                                    $erro[] = $this->TAM_UNIDADE;
                                } else {
                                    if ($this->verificaUnidadeMedida($unidade) == 0) {
                                        $erro[] = $this->UN_PROD_N_EXISTE;
                                    } else if ($prodExiste['SGUNIDADE'] <> $unidade) 
                                    {
                                        $erro[] = $this->UN_PROD_DIF;
                                    }
                                }
                            } else $erro[] = $this->UN_PRODUTO_N_INFO;
                        }
                    }
                    //valida quantidade do produto
                    if($qtd == null){
                        $erro[] = $this->QTD_PROD_N_INFO;
                    } else if (!(is_float($qtd) OR is_numeric($qtd)) OR ($qtd) < 0) {        
                        $erro[] = $this->QTD_PROD_INVALIDA;
                    } else {
                        $qtd = number_format($qtd, 5, ',', '');
                    }
                    
                } else {
                    $produto = null;
                    $unidade = null;
                    $qtd = null;
                }
                    
                $value = array('CDPRODUTO' => $produto, 'SGUNIPROD' => $unidade, 'QTPRODUTO' => $qtd);
                
                break;
            case 'NMPRATO': 
                if (strlen($value) == 0) $erro[] = $this->NMPRATO_N_INFO; 
                else if (strlen($value) > 40) {
                    $erro[] = $this->TAM_PRATO;
                }

                break;
            case 'SGUNIDPRATO':
                $value = strtoupper($value);
                
                if (empty($value) || strlen($value) == 0) $erro[] = $this->UN_PRATO_N_INFO; 
                else if (strlen($value) > 10) {
                    $erro[] = $this->TAM_UNIDADE;
                } else {
                    if ($this->verificaUnidadeMedida($value) == 0) {
                        $erro[] = $this->UN_PRATO_N_EXISTE;
                    }
                }
                break;
            case 'QTPERDA': 
                if($this->isNumeric($value)){
                    $value = $this->valueToFloat($value);
                }
                if($value == null){
                    $value = '0,00000';
                } elseif (!(is_float($value) OR is_numeric($value))) {        
                    $erro[] = $this->QTD_PERDA_INVALIDA;
                } else if($value > 100 OR $value < 0){
                    $erro[] = $this->QTD_SUPERIOR;
                } else {
                    $value = number_format($value, 5, ',', '');
                }
                break;
            default: 
            $value = strval($value);
        }
        
        if (count($erro) > 0) throw new \Exception(json_encode($erro));
        else return $value;
    }
    
    protected function verificaUnidadeMedida($unidade){
        $sql = "SELECT COUNT(1) EXISTE FROM UNIDMEDIDA WHERE SGUNIDADE = :SGUNIDADE";
        $params = array(
            "SGUNIDADE" => $unidade
        );
        
        $query  = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchColumn();
        return $result;
    }
    
    protected function verificaProduto($cdProduto){
        $sql = "SELECT CDPRODUTO, SGUNIDADE FROM PRODUTO WHERE CDPRODUTO = :CDPRODUTO AND CDPRODUTO = CDPRODESTO";
        $params = array(
            "CDPRODUTO" => $cdProduto
        );
        
        $query  = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
    }
    
    protected function verificaMascPrato($value){
        $query = "SELECT CDPICTPRAT FROM PARAMGERAL";
        $connection =  $this->entityManager->getConnection();
        $stmt = $connection->prepare($query);
        $stmt->execute();
        $mascara = $stmt->fetch()['CDPICTPRAT'];
        
        if ($mascara) {
            $mascara = explode('.', $mascara);
            $tamPosAnt = 0;
            
            foreach ($mascara as $parteMasc) {
                $tamPos = strlen($parteMasc);
                $valor = substr($value, $tamPosAnt, $tamPos);
                
                if (strlen($valor) > 0 and strlen($valor) <> $tamPos) {
                    return false;
                }
                
                $tamPosAnt = $tamPosAnt + $tamPos;
            }
        } else return $this->MASC_N_DEFINIDA;
        
        return true;
    }
    
     public function updateProdutoRecefili($prodASubstituirLocal, $prodNovoLocal, $prodFilial , $filAuxPrato, $prato) {
        $update = "UPDATE RECEFILI
                    SET
                        CDPRODUTO = :CDPRODUTOAUX
                    WHERE
                            CDFILAUXPRAT = :CDFILAUXPRAT
                        AND CDPRATO = :CDPRATO
                        AND CDFILIAL = :CDFILIAL
                        AND CDPRODUTO = :CDPRODUTO";
        $params = array(
            "CDPRODUTOAUX" => $prodNovoLocal,
            "CDFILAUXPRAT" => $filAuxPrato,
            "CDPRATO" => $prato,
            "CDFILIAL" => $prodFilial,
            "CDPRODUTO" => $prodASubstituirLocal,
        );
        
        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->execute($params);

    }
    
    public function updateProdutoRecepadr($prodASubstituirPadrao, $prodNovoPadrao , $filAuxPrato, $prato) {
        $update = "UPDATE RECEPADR
                    SET
                        CDPRODUTO = :CDPRODUTOAUX
                    WHERE
                            CDFILAUXPRAT = :CDFILAUXPRAT
                        AND CDPRATO = :CDPRATO
                        AND CDPRODUTO = :CDPRODUTO";
        $params = array(
            "CDPRODUTOAUX" => $prodNovoPadrao,
            "CDFILAUXPRAT" => $filAuxPrato,
            "CDPRATO" => $prato,
            "CDPRODUTO" => $prodASubstituirPadrao,
        );
        
        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->execute($params);

    }
    
     public function updateProdutoReceserv($prodASubstituirServico, $prodNovoServico, $prodFilial , $filAuxPrato, $prato, $prodCdservfili) {
        $update = "UPDATE RECESERV
                    SET
                        CDPRODUTO = :CDPRODUTOAUX
                    WHERE
                            CDFILAUXPRAT = :CDFILAUXPRAT
                        AND CDPRATO = :CDPRATO
                        AND CDFILIAL = :CDFILIAL
                        AND CDPRODUTO = :CDPRODUTO
                        AND CDSERVFILI = :CDSERVFILI";
        $params = array(
            "CDPRODUTOAUX" => $prodNovoServico,
            "CDFILAUXPRAT" => $filAuxPrato,
            "CDPRATO" => $prato,
            "CDFILIAL" => $prodFilial,
            "CDPRODUTO" => $prodASubstituirServico,
            "CDSERVFILI" => $prodCdservfili,
        );
        
        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->execute($params);

    }
    
    public function updateEmbAtual($dadosUpdateEmb, $CdcontainerAntigo){

        try {
            $update = "UPDATE PRATOCONTAINER 
                        SET 
                            CDCONTAINER = :CDCONTAINER_NOVO,
                            QTPRATCONTAINER = :QTPRATCONTAINER,
                            IDPRATCONTPRI = :IDPRATCONTPRI,
                            NRORG = :NRORG,
                            DTINCLUSAO = :DTINCLUSAO,
                            DTULTATU = :DTULTATU,
                            NRORGINCLUSAO = :NRORGINCLUSAO,
                            CDOPERINCLUSAO = :CDOPERINCLUSAO,
                            NRORGULTATU = :NRORGULTATU,
                            CDOPERULTATU = :CDOPERULTATU,
                            IDATIVO = :IDATIVO
                    WHERE 
                            CDFILAUXPRAT = :CDFILAUXPRAT
                        AND CDPRATO = :CDPRATO
                        AND CDCONTAINER = :CDCONTAINER_ANTIGO";
            $params = array(
                "CDCONTAINER_NOVO" => $dadosUpdateEmb[0]['CDCONTAINER'],
                "CDCONTAINER_ANTIGO" => $CdcontainerAntigo,
                "CDPRATO" => $dadosUpdateEmb[0]['CDPRATO'],
                "CDFILAUXPRAT" => $dadosUpdateEmb[0]['CDFILAUXPRAT'],
                "QTPRATCONTAINER" => $dadosUpdateEmb[0]['QTPRATCONTAINER'],
                "IDPRATCONTPRI" => $dadosUpdateEmb[0]['IDPRATCONTPRI'],
                "NRORG" => $dadosUpdateEmb[0]['NRORG'],
                "DTINCLUSAO" => $dadosUpdateEmb[0]['DTINCLUSAO'],
                "DTULTATU" => $dadosUpdateEmb[0]['DTULTATU'],
                "NRORGINCLUSAO" => $dadosUpdateEmb[0]['NRORGINCLUSAO'],
                "CDOPERINCLUSAO" => $dadosUpdateEmb[0]['CDOPERINCLUSAO'],
                "NRORGULTATU" => $dadosUpdateEmb[0]['NRORGULTATU'],
                "CDOPERULTATU" => $dadosUpdateEmb[0]['CDOPERULTATU'],
                "IDATIVO" => $dadosUpdateEmb[0]['IDATIVO'],
                
            );
            
            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->execute($params);
           
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function verificaEmbExistente($cdcontainer, $cdprato, $cdfilauxprat){
     
        $sql = "SELECT CDCONTAINER FROM PRATOCONTAINER WHERE CDCONTAINER = :CDCONTAINER AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT";
        $params = array(
            "CDCONTAINER" => $cdcontainer,
            "CDPRATO" => $cdprato,
            "CDFILAUXPRAT" => $cdfilauxprat
        );
        
        $query  = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
        
    }
    
    
    
    
    
} 